import asyncio
import json
import time
import logging
from datetime import datetime, timedelta, timezone
from typing import Dict, List, Optional, Any
from contextlib import asynccontextmanager
from fastapi import FastAPI, WebSocket, WebSocketDisconnect, Query
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from starlette.websockets import WebSocketState
from redis import asyncio as aioredis
import uvicorn
from pathlib import Path
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from sqlalchemy import select, func, and_, or_, text
from sqlalchemy.dialects import postgresql

from jettask.webui.pg_consumer import PostgreSQLConsumer
from jettask.webui.config import PostgreSQLConfig, RedisConfig
from jettask.webui.models import Base, Task, QueueStats, Worker

logger = logging.getLogger(__name__)

# SQLAlchemy异步引擎和会话（独立于consumer）
async_engine = None
AsyncSessionLocal = None

def parse_iso_datetime(time_str: str) -> datetime:
    """解析ISO格式的时间字符串，确保返回 UTC 时间"""
    if time_str.endswith('Z'):
        # Z 表示 UTC 时间
        dt = datetime.fromisoformat(time_str.replace('Z', '+00:00'))
    else:
        dt = datetime.fromisoformat(time_str)
    
    # 如果没有时区信息，假定为 UTC
    if dt.tzinfo is None:
        dt = dt.replace(tzinfo=timezone.utc)
    # 如果有时区信息，转换为 UTC
    elif dt.tzinfo != timezone.utc:
        dt = dt.astimezone(timezone.utc)
    
    return dt

async def get_db_engine():
    """获取SQLAlchemy异步引擎（用于读取数据）"""
    global async_engine, AsyncSessionLocal
    
    if async_engine:
        return async_engine
    
    # 尝试从环境变量或配置获取PostgreSQL连接信息
    import os
    
    pg_config = PostgreSQLConfig.from_env()
    
    if not pg_config.dsn:
        logger.warning("PostgreSQL connection not configured")
        return None
    
    try:
        # 将 DSN 转换为 SQLAlchemy 格式
        # 如果是 postgresql:// 开头，改为 postgresql+psycopg://
        if pg_config.dsn.startswith('postgresql://'):
            dsn = pg_config.dsn.replace('postgresql://', 'postgresql+psycopg://', 1)
        else:
            dsn = pg_config.dsn
            
        async_engine = create_async_engine(
            dsn,
            pool_size=10,
            max_overflow=5,
            pool_pre_ping=True,
            echo=False
        )
        
        # 创建异步会话工厂
        AsyncSessionLocal = sessionmaker(
            async_engine,
            class_=AsyncSession,
            expire_on_commit=False
        )
        
        logger.info("SQLAlchemy async engine created for WebUI")
        return async_engine
    except Exception as e:
        logger.error(f"Failed to create SQLAlchemy async engine: {e}")
        return None

async def get_db_session():
    """获取数据库会话"""
    if not AsyncSessionLocal:
        await get_db_engine()
    
    if AsyncSessionLocal:
        async with AsyncSessionLocal() as session:
            yield session
    else:
        yield None

class RedisMonitor:
    def __init__(self, redis_url: str = "redis://localhost:6379", redis_prefix: str = "jettask"):
        self.redis_url = redis_url
        self.redis_prefix = redis_prefix
        self.redis: Optional[aioredis.Redis] = None
        self.scanner_task: Optional[asyncio.Task] = None
        self.scanner_interval = 5  # 5秒扫描一次
        self.default_heartbeat_timeout = 30  # 默认30秒心跳超时
        self._queues_cache = None
        self._queues_cache_time = 0
        self._queues_cache_ttl = 60  # 缓存60秒
        self._workers_cache = None
        self._workers_cache_time = 0
        self._workers_cache_ttl = 5  # worker缓存5秒，因为更新频繁
        self._scanner_running = False  # 标记扫描器是否正在运行
        
    async def connect(self):
        # 使用项目中的连接池模式，优化连接参数
        pool = aioredis.ConnectionPool.from_url(
            self.redis_url,
            decode_responses=True,
            max_connections=100,  # 增加最大连接数
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_connect_timeout=5,  # 减少连接超时时间
            socket_timeout=10,  # 设置socket超时
            health_check_interval=30
        )
        self.redis = aioredis.Redis(connection_pool=pool)
        
    async def close(self):
        # 停止扫描器任务
        if self.scanner_task and not self.scanner_task.done():
            self.scanner_task.cancel()
            try:
                await self.scanner_task
            except asyncio.CancelledError:
                pass
        
        if self.redis:
            await self.redis.close()
    
    def get_prefixed_queue_name(self, queue_name: str) -> str:
        """为队列名称添加前缀"""
        return f"{self.redis_prefix}:QUEUE:{queue_name}"
  
            
    async def get_task_info(self, event_id: str) -> Dict[str, Any]:
        """获取任务详细信息"""
        status_key = f"{self.redis_prefix}:STATUS:{event_id}"
        result_key = f"{self.redis_prefix}:RESULT:{event_id}"
        
        status = await self.redis.get(status_key)
        result = await self.redis.get(result_key)
        
        task_info = {
            "event_id": event_id,
            "status": status,
            "result": result
        }
        
        # 如果有状态信息，尝试从对应的队列stream中获取详细信息
        if status:
            try:
                status_data = json.loads(status)
                queue_name = status_data.get("queue")
                
                if queue_name:
                    # 从stream中查找该任务
                    # 使用 xrange 扫描最近的消息
                    prefixed_queue_name = self.get_prefixed_queue_name(queue_name)
                    messages = await self.redis.xrange(prefixed_queue_name, count=1000)
                    
                    for msg_id, data in messages:
                        # 检查消息数据中的event_id是否匹配
                        if (data.get("event_id") == event_id or 
                            data.get("id") == event_id or
                            data.get("task_id") == event_id):
                            task_info["stream_data"] = {
                                "message_id": msg_id,
                                "data": data,
                                "queue": queue_name
                            }
                            break
                    
                    # 如果消息ID就是event_id，直接尝试获取
                    if not task_info.get("stream_data"):
                        try:
                            direct_messages = await self.redis.xrange(
                                prefixed_queue_name, 
                                min=event_id, 
                                max=event_id, 
                                count=1
                            )
                            if direct_messages:
                                msg_id, data = direct_messages[0]
                                task_info["stream_data"] = {
                                    "message_id": msg_id,
                                    "data": data,
                                    "queue": queue_name
                                }
                        except:
                            pass
                            
            except Exception as e:
                print(f"Error parsing status for task {event_id}: {e}")
                
        return task_info
        
    async def get_stream_info(self, queue_name: str, event_id: str) -> Optional[Dict[str, Any]]:
        """从Stream中获取任务详细信息"""
        try:
            prefixed_queue_name = self.get_prefixed_queue_name(queue_name)
            # 先尝试按event_id直接查找
            messages = await self.redis.xrange(prefixed_queue_name, min=event_id, max=event_id, count=1)
            if messages:
                msg_id, data = messages[0]
                return {
                    "message_id": msg_id,
                    "data": data,
                    "queue": queue_name
                }
            
            # 如果没找到，可能event_id是消息内容的一部分，扫描最近的消息
            messages = await self.redis.xrange(prefixed_queue_name, count=100)
            for msg_id, data in messages:
                if data.get("event_id") == event_id or data.get("id") == event_id:
                    return {
                        "message_id": msg_id,
                        "data": data,
                        "queue": queue_name
                    }
        except Exception as e:
            print(f"Error reading from stream {prefixed_queue_name}: {e}")
        return None
        
    async def get_all_tasks(self) -> List[Dict[str, Any]]:
        """获取所有任务状态"""
        tasks = []
        
        # 扫描所有状态键
        cursor = 0
        pattern = f"{self.redis_prefix}:STATUS:*"
        
        while True:
            cursor, keys = await self.redis.scan(
                cursor, match=pattern, count=100
            )
            
            for status_key in keys:
                event_id = status_key.split(":")[-1]
                task_info = await self.get_task_info(event_id)
                
                # 尝试获取Stream信息
                # 这里需要知道任务所在的队列，可能需要从状态信息中解析
                if task_info.get("status"):
                    try:
                        status_data = json.loads(task_info["status"])
                        queue_name = status_data.get("queue")
                        if queue_name:
                            stream_info = await self.get_stream_info(queue_name, event_id)
                            if stream_info:
                                task_info["stream_info"] = stream_info
                    except:
                        pass
                        
                tasks.append(task_info)
                
            if cursor == 0:
                break
                
        return tasks
    
    async def get_queue_tasks(self, queue_name: str, start_time: Optional[str] = None, 
                             end_time: Optional[str] = None, limit: int = 100) -> Dict[str, Any]:
        """获取指定队列的任务（基于时间范围）
        
        Args:
            queue_name: 队列名称
            start_time: 开始时间（Redis Stream ID格式或时间戳）
            end_time: 结束时间（Redis Stream ID格式或时间戳）
            limit: 返回的最大任务数
        """
        all_tasks = []
        
        try:
            # 处理时间参数
            # 如果没有指定结束时间，使用 '+' 表示到最新
            if not end_time:
                end_time = '+'
            
            # 如果没有指定开始时间，使用 '-' 表示从最早开始
            if not start_time:
                start_time = '-'
                
            # 从队列的stream中读取消息
            # 使用 xrevrange 按时间倒序获取（最新的在前）
            prefixed_queue_name = self.get_prefixed_queue_name(queue_name)
            messages = await self.redis.xrevrange(
                prefixed_queue_name, 
                max=end_time,
                min=start_time,
                count=limit
            )
            
            for msg_id, data in messages:
                # 在easy_task中，event_id就是Redis生成的stream消息ID
                event_id = msg_id
                
                # 构建任务信息
                task_info = {
                    "event_id": event_id,
                    "message_id": msg_id,
                    "stream_data": data,
                    "task_name": data.get("name", "unknown"),
                    "queue": data.get("queue", queue_name),
                    "trigger_time": data.get("trigger_time")
                }
                
                # 尝试解析args和kwargs，并组合成参数字符串
                params_str = ""
                try:
                    args_list = []
                    kwargs_dict = {}
                    
                    if data.get("args"):
                        args_list = json.loads(data["args"])
                        task_info["args"] = args_list
                    
                    if data.get("kwargs"):
                        kwargs_dict = json.loads(data["kwargs"])
                        task_info["kwargs"] = kwargs_dict
                    
                    # 构建参数字符串
                    params_parts = []
                    if args_list:
                        params_parts.extend([str(arg) for arg in args_list])
                    if kwargs_dict:
                        params_parts.extend([f"{k}={v}" for k, v in kwargs_dict.items()])
                    
                    params_str = ", ".join(params_parts) if params_parts else "无参数"
                    
                except Exception as e:
                    params_str = "解析失败"
                    
                task_info["params_str"] = params_str
                
                # 从状态键获取信息（不默认获取结果）
                status_key = f"{self.redis_prefix}:STATUS:{event_id}"
                
                # 获取状态
                status = await self.redis.get(status_key)
                
                if status:
                    task_info["status"] = status
                    try:
                        parsed_status = json.loads(status)
                        task_info["parsed_status"] = parsed_status
                        # 从状态中获取消费者信息
                        task_info["consumer"] = parsed_status.get("consumer", "-")
                    except:
                        task_info["parsed_status"] = {"status": "unknown"}
                        task_info["consumer"] = "-"
                else:
                    # 如果没有状态，显示未知
                    task_info["status"] = json.dumps({
                        "status": "未知", 
                        "queue": queue_name,
                        "created_at": datetime.fromtimestamp(float(data.get("trigger_time", 0))).isoformat() if data.get("trigger_time") else None
                    })
                    task_info["parsed_status"] = {
                        "status": "未知", 
                        "queue": queue_name,
                        "created_at": datetime.fromtimestamp(float(data.get("trigger_time", 0))).isoformat() if data.get("trigger_time") else None
                    }
                    task_info["consumer"] = "-"
                
                all_tasks.append(task_info)
                
        except Exception as e:
            print(f"Error reading queue {queue_name}: {e}")
            # 如果stream不存在或出错，返回空结果
            return {
                "tasks": [],
                "count": 0,
                "oldest_id": None,
                "newest_id": None,
                "has_more": False,
                "limit": limit
            }
        
        # 获取最早和最晚的消息ID用于分页导航
        oldest_id = all_tasks[-1]["message_id"] if all_tasks else None
        newest_id = all_tasks[0]["message_id"] if all_tasks else None
        
        # 检查是否还有更多数据
        has_more = len(messages) >= limit
        
        # 获取队列总长度
        total_count = 0
        try:
            queue_info = await self.redis.xinfo_stream(prefixed_queue_name)
            total_count = queue_info.get("length", 0)
        except Exception as e:
            print(f"Error getting queue info for {queue_name}: {e}")
            total_count = len(all_tasks)
        
        return {
            "tasks": all_tasks,
            "count": len(all_tasks),
            "total_count": total_count,
            "oldest_id": oldest_id,
            "newest_id": newest_id,
            "has_more": has_more,
            "limit": limit
        }
        
    async def get_worker_heartbeats(self, queue_name: str) -> List[Dict[str, Any]]:
        """获取指定队列的Worker心跳信息 - 直接扫描WORKER键"""
        worker_list = []
        current_time = datetime.now(timezone.utc).timestamp()
        
        # 直接扫描所有WORKER键（排除HISTORY相关的键）
        pattern = f"{self.redis_prefix}:WORKER:*"
        cursor = 0
        worker_keys = []
        
        while True:
            cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)
            # 过滤掉HISTORY相关的键
            for key in keys:
                if ':HISTORY:' not in key:
                    worker_keys.append(key)
            if cursor == 0:
                break
        
        # 批量获取所有worker数据
        if worker_keys:
            pipe = self.redis.pipeline()
            for key in worker_keys:
                pipe.hgetall(key)
            all_workers_data = await pipe.execute()
            
            for i, worker_data in enumerate(all_workers_data):
                if not worker_data:
                    continue
                
                # 检查worker是否属于指定队列
                worker_queues = worker_data.get('queues', '')
                if queue_name not in worker_queues.split(','):
                    continue
                
                worker_id = worker_keys[i].split(':')[-1]
                last_heartbeat = float(worker_data.get('last_heartbeat', 0))
                is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                consumer_id = worker_data.get('consumer_id', worker_id)
                
                # 构建显示数据
                display_data = {
                    'consumer_id': consumer_id,
                    'consumer_name': f"{consumer_id}-{queue_name}",  # 保持兼容性
                    'host': worker_data.get('host', 'unknown'),
                    'pid': int(worker_data.get('pid', 0)),
                    'queue': queue_name,
                    'last_heartbeat': last_heartbeat,
                    'last_heartbeat_time': datetime.fromtimestamp(last_heartbeat).isoformat(),
                    'seconds_ago': int(current_time - last_heartbeat),
                    'is_alive': is_alive,
                    # 队列特定的统计信息
                    'success_count': int(worker_data.get(f'{queue_name}:success_count', 0)),
                    'failed_count': int(worker_data.get(f'{queue_name}:failed_count', 0)),
                    'total_count': int(worker_data.get(f'{queue_name}:total_count', 0)),
                    'running_tasks': int(worker_data.get(f'{queue_name}:running_tasks', 0)),
                    'avg_processing_time': float(worker_data.get(f'{queue_name}:avg_processing_time', 0.0)),
                    'avg_latency_time': float(worker_data.get(f'{queue_name}:avg_latency_time', 0.0))
                }
                
                # 如果离线时间存在，添加离线时间信息
                if 'offline_time' in worker_data:
                    display_data['offline_time'] = float(worker_data['offline_time'])
                    display_data['offline_time_formatted'] = datetime.fromtimestamp(float(worker_data['offline_time'])).isoformat()
                
                worker_list.append(display_data)
                
        return worker_list
    
    async def get_queue_worker_summary(self, queue_name: str) -> Dict[str, Any]:
        """获取队列的worker汇总统计信息"""
        try:
            # 直接扫描所有WORKER键并过滤（排除HISTORY相关的键）
            pattern = f"{self.redis_prefix}:WORKER:*"
            cursor = 0
            worker_keys = []
            
            while True:
                cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)
                # 过滤掉HISTORY相关的键
                for key in keys:
                    if ':HISTORY:' not in key:
                        worker_keys.append(key)
                if cursor == 0:
                    break
            
            if not worker_keys:
                return {
                    'total_workers': 0,
                    'online_workers': 0,
                    'offline_workers': 0,
                    'total_success_count': 0,
                    'total_failed_count': 0,
                    'total_count': 0,
                    'total_running_tasks': 0,
                    'avg_processing_time': 0.0,
                'avg_latency_time': 0.0
                }
            
            # 批量获取worker数据
            pipe = self.redis.pipeline()
            for key in worker_keys:
                pipe.hgetall(key)
            all_workers_data = await pipe.execute()
            
            # 过滤属于该队列的worker
            queue_workers_data = []
            for i, worker_data in enumerate(all_workers_data):
                if worker_data and queue_name in worker_data.get('queues', '').split(','):
                    queue_workers_data.append(worker_data)
            
            # 汇总统计
            total_workers = len(queue_workers_data)
            online_workers = 0
            offline_workers = 0
            total_success_count = 0
            total_failed_count = 0
            total_count = 0
            total_running_tasks = 0
            total_processing_time = 0.0
            processing_time_count = 0
            total_latency_time = 0.0
            latency_time_count = 0
            
            current_time = datetime.now(timezone.utc).timestamp()
            offline_worker_ids = []  # 记录离线worker的ID，避免从历史中重复统计
            
            for worker_data in queue_workers_data:
                try:
                    # 检查worker状态
                    last_heartbeat = float(worker_data.get('last_heartbeat', 0))
                    is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                    worker_id = worker_data.get('consumer_id', '')
                    
                    if is_alive and (current_time - last_heartbeat) < 30:
                        online_workers += 1
                        # 只统计在线worker的数据
                        success_count = int(worker_data.get(f'{queue_name}:success_count', 0))
                        failed_count = int(worker_data.get(f'{queue_name}:failed_count', 0))
                        running_tasks = int(worker_data.get(f'{queue_name}:running_tasks', 0))
                        avg_processing_time = float(worker_data.get(f'{queue_name}:avg_processing_time', 0.0))
                        avg_latency_time = float(worker_data.get(f'{queue_name}:avg_latency_time', 0.0))
                        
                        total_success_count += success_count
                        total_failed_count += failed_count
                        total_count += success_count + failed_count
                        total_running_tasks += running_tasks
                        
                        if avg_processing_time > 0:
                            total_processing_time += avg_processing_time
                            processing_time_count += 1
                        
                        if avg_latency_time > 0:
                            total_latency_time += avg_latency_time
                            latency_time_count += 1
                    else:
                        offline_workers += 1
                        # 记录离线worker的ID，从历史中统计
                        if worker_id:
                            offline_worker_ids.append(worker_id)
                        
                except Exception as e:
                    print(f"Error processing worker summary: {e}")
                    continue
            
            # 统计离线worker的数据（从WORKER键中）
            for worker_data in queue_workers_data:
                try:
                    # 检查是否是离线worker
                    is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                    worker_id = worker_data.get('consumer_id', '')
                    
                    if not is_alive and worker_id in offline_worker_ids:
                        # 统计离线worker的数据
                        success_count = int(worker_data.get(f'{queue_name}:success_count', 0))
                        failed_count = int(worker_data.get(f'{queue_name}:failed_count', 0))
                        
                        total_success_count += success_count
                        total_failed_count += failed_count
                        total_count += success_count + failed_count
                        
                        # 处理时间统计
                        avg_processing_time = float(worker_data.get(f'{queue_name}:avg_processing_time', 0.0))
                        if avg_processing_time > 0:
                            total_processing_time += avg_processing_time
                            processing_time_count += 1
                        
                        # 延迟时间统计
                        avg_latency_time = float(worker_data.get(f'{queue_name}:avg_latency_time', 0.0))
                        if avg_latency_time > 0:
                            total_latency_time += avg_latency_time
                            latency_time_count += 1
                            
                except Exception as e:
                    print(f"Error processing offline worker stats: {e}")
                    continue
            
            # 计算平均处理时间（包含历史）
            overall_avg_processing_time = 0.0
            if processing_time_count > 0:
                overall_avg_processing_time = total_processing_time / processing_time_count
            
            # 计算平均延迟时间
            overall_avg_latency_time = 0.0
            if latency_time_count > 0:
                overall_avg_latency_time = total_latency_time / latency_time_count
            
            return {
                'total_workers': total_workers,
                'online_workers': online_workers,
                'offline_workers': offline_workers,
                'total_success_count': total_success_count,
                'total_failed_count': total_failed_count,
                'total_count': total_count,
                'total_running_tasks': total_running_tasks,
                'avg_processing_time': round(overall_avg_processing_time, 3),
                'avg_latency_time': round(overall_avg_latency_time, 3),
                'history_included': True
            }
            
        except Exception as e:
            print(f"Error getting queue worker summary for {queue_name}: {e}")
            return {
                'total_workers': 0,
                'online_workers': 0,
                'offline_workers': 0,
                'total_success_count': 0,
                'total_failed_count': 0,
                'total_count': 0,
                'total_running_tasks': 0,
                'avg_processing_time': 0.0,
                'avg_latency_time': 0.0
            }
    
    async def get_queue_worker_summary_fast(self, queue_name: str) -> Dict[str, Any]:
        """获取队列的worker汇总统计信息（快速版，不包含历史）"""
        try:
            # 直接扫描所有WORKER键（排除HISTORY相关的键）
            pattern = f"{self.redis_prefix}:WORKER:*"
            cursor = 0
            worker_keys = []
            
            while True:
                cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)
                # 过滤掉HISTORY相关的键
                for key in keys:
                    if ':HISTORY:' not in key:
                        worker_keys.append(key)
                if cursor == 0:
                    break
            
            if not worker_keys:
                return {
                    'total_workers': 0,
                    'online_workers': 0,
                    'offline_workers': 0,
                    'total_success_count': 0,
                    'total_failed_count': 0,
                    'total_count': 0,
                    'total_running_tasks': 0,
                    'avg_processing_time': 0.0,
                'avg_latency_time': 0.0
                }
            
            # 使用pipeline批量获取worker数据
            pipe = self.redis.pipeline()
            for worker_key in worker_keys:
                pipe.hgetall(worker_key)
            
            all_workers_data = await pipe.execute()
            
            # 过滤属于该队列的worker
            worker_data_list = []
            for worker_data in all_workers_data:
                if worker_data and queue_name in worker_data.get('queues', '').split(','):
                    worker_data_list.append(worker_data)
            
            # 汇总统计
            total_workers = len(worker_data_list)
            online_workers = 0
            offline_workers = 0
            total_success_count = 0
            total_failed_count = 0
            total_count = 0
            total_running_tasks = 0
            total_processing_time = 0.0
            processing_time_count = 0
            total_latency_time = 0.0
            latency_time_count = 0
            
            current_time = datetime.now(timezone.utc).timestamp()
            
            for worker_data in worker_data_list:
                if not worker_data:
                    continue
                
                # 检查worker状态
                last_heartbeat = float(worker_data.get('last_heartbeat', 0))
                is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                
                if is_alive and (current_time - last_heartbeat) < 30:
                    online_workers += 1
                    # 只统计在线worker的数据（快速版不包含历史，所以只统计在线的）
                    success_count = int(worker_data.get(f'{queue_name}:success_count', 0))
                    failed_count = int(worker_data.get(f'{queue_name}:failed_count', 0))
                    running_tasks = int(worker_data.get(f'{queue_name}:running_tasks', 0))
                    avg_processing_time = float(worker_data.get(f'{queue_name}:avg_processing_time', 0.0))
                    avg_latency_time = float(worker_data.get(f'{queue_name}:avg_latency_time', 0.0))
                    
                    total_success_count += success_count
                    total_failed_count += failed_count
                    total_count += success_count + failed_count
                    total_running_tasks += running_tasks
                    
                    if avg_processing_time > 0:
                        total_processing_time += avg_processing_time
                        processing_time_count += 1
                    
                    if avg_latency_time > 0:
                        total_latency_time += avg_latency_time
                        latency_time_count += 1
                else:
                    offline_workers += 1
                    # 快速版不统计离线worker的数据
            
            # 计算平均处理时间
            avg_processing_time = 0.0
            if processing_time_count > 0:
                avg_processing_time = total_processing_time / processing_time_count
            
            # 计算平均延迟时间
            avg_latency_time = 0.0
            if latency_time_count > 0:
                avg_latency_time = total_latency_time / latency_time_count
            
            return {
                'total_workers': total_workers,
                'online_workers': online_workers,
                'offline_workers': offline_workers,
                'total_success_count': total_success_count,
                'total_failed_count': total_failed_count,
                'total_count': total_count,
                'total_running_tasks': total_running_tasks,
                'avg_processing_time': round(avg_processing_time, 3),
                'avg_latency_time': round(avg_latency_time, 3)
            }
            
        except Exception as e:
            print(f"Error getting queue worker summary for {queue_name}: {e}")
            return {
                'total_workers': 0,
                'online_workers': 0,
                'offline_workers': 0,
                'total_success_count': 0,
                'total_failed_count': 0,
                'total_count': 0,
                'total_running_tasks': 0,
                'avg_processing_time': 0.0,
                'avg_latency_time': 0.0
            }
    
    async def get_worker_offline_history(self, limit: int = 100, start_time: Optional[float] = None, end_time: Optional[float] = None) -> List[Dict[str, Any]]:
        """获取worker下线历史记录 - 直接从WORKER键读取离线worker信息"""
        try:
            # 扫描所有WORKER键（排除HISTORY相关的键）
            pattern = f"{self.redis_prefix}:WORKER:*"
            cursor = 0
            worker_keys = []
            
            while True:
                cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)
                # 过滤掉HISTORY相关的键（虽然已经不再创建，但以防万一）
                for key in keys:
                    if ':HISTORY:' not in key:
                        worker_keys.append(key)
                if cursor == 0:
                    break
            
            if not worker_keys:
                return []
            
            # 批量获取所有worker数据
            pipe = self.redis.pipeline()
            for key in worker_keys:
                pipe.hgetall(key)
            all_workers_data = await pipe.execute()
            
            # 收集离线的worker
            offline_workers = []
            current_time = time.time()
            
            for i, worker_data in enumerate(all_workers_data):
                if not worker_data:
                    continue
                
                # 检查是否离线
                is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                if not is_alive and 'offline_time' in worker_data:
                    offline_time = float(worker_data.get('offline_time', 0))
                    
                    # 时间范围过滤
                    if start_time and offline_time < start_time:
                        continue
                    if end_time and offline_time > end_time:
                        continue
                    
                    # 计算运行时长
                    online_time = float(worker_data.get('created_at', offline_time))
                    duration_seconds = int(offline_time - online_time)
                    
                    # 构建离线记录
                    record = {
                        'consumer_id': worker_data.get('consumer_id', ''),
                        'host': worker_data.get('host', 'unknown'),
                        'pid': int(worker_data.get('pid', 0)),
                        'queues': worker_data.get('queues', ''),
                        'online_time': online_time,
                        'offline_time': offline_time,
                        'duration_seconds': duration_seconds,
                        'last_heartbeat': float(worker_data.get('last_heartbeat', 0)),
                        'shutdown_reason': worker_data.get('shutdown_reason', 'unknown'),
                        'online_time_str': datetime.fromtimestamp(online_time).isoformat(),
                        'offline_time_str': datetime.fromtimestamp(offline_time).isoformat(),
                    }
                    
                    # 格式化运行时长
                    hours = duration_seconds // 3600
                    minutes = (duration_seconds % 3600) // 60
                    seconds = duration_seconds % 60
                    record['duration_str'] = f"{hours}h {minutes}m {seconds}s"
                    
                    # 添加统计信息（聚合所有队列的数据）
                    queues = worker_data.get('queues', '').split(',') if worker_data.get('queues') else []
                    total_success = 0
                    total_failed = 0
                    total_count = 0
                    
                    for queue in queues:
                        if queue.strip():
                            queue = queue.strip()
                            total_success += int(worker_data.get(f'{queue}:success_count', 0))
                            total_failed += int(worker_data.get(f'{queue}:failed_count', 0))
                            total_count += int(worker_data.get(f'{queue}:total_count', 0))
                    
                    record['total_success_count'] = total_success
                    record['total_failed_count'] = total_failed
                    record['total_count'] = total_count
                    record['total_running_tasks'] = 0  # 离线worker没有运行中的任务
                    
                    # 计算平均处理时间
                    if total_count > 0:
                        total_processing_time = 0.0
                        for queue in queues:
                            if queue.strip():
                                queue = queue.strip()
                                avg_time = float(worker_data.get(f'{queue}:avg_processing_time', 0))
                                count = int(worker_data.get(f'{queue}:total_count', 0))
                                if avg_time > 0 and count > 0:
                                    total_processing_time += avg_time * count
                        record['avg_processing_time'] = total_processing_time / total_count
                    else:
                        record['avg_processing_time'] = 0.0
                    
                    offline_workers.append((offline_time, record))
            
            # 按离线时间倒序排序
            offline_workers.sort(key=lambda x: x[0], reverse=True)
            
            # 返回指定数量的记录
            return [record for _, record in offline_workers[:limit]]
            
        except Exception as e:
            print(f"Error getting worker offline history: {e}")
            return []
    
    async def get_global_stats_with_history(self) -> Dict[str, Any]:
        """获取全局统计信息（优化版）- 注：不再重复统计历史数据"""
        try:
            # 获取所有队列
            queues = await self.get_all_queues()
            
            # 并行获取所有队列的汇总信息和队列统计
            queue_summaries_task = asyncio.gather(
                *[self.get_queue_worker_summary_fast(queue) for queue in queues],
                return_exceptions=True
            )
            queue_stats_task = asyncio.gather(
                *[self.get_queue_stats(queue) for queue in queues],
                return_exceptions=True
            )
            
            queue_summaries, queue_stats = await asyncio.gather(
                queue_summaries_task, queue_stats_task
            )
            
            # 初始化统计
            total_success = 0
            total_failed = 0
            total_tasks = 0
            total_running = 0
            total_workers = 0
            online_workers = 0
            offline_workers = 0
            total_processing_time = 0.0
            total_processing_count = 0
            total_latency_time = 0.0
            total_latency_count = 0
            
            # RabbitMQ风格指标
            total_messages = 0
            total_messages_ready = 0
            total_messages_unacknowledged = 0
            total_consumers = 0
            total_publish = 0
            total_deliver_get = 0
            total_ack = 0
            
            # 汇总统计信息
            for i, summary in enumerate(queue_summaries):
                if isinstance(summary, Exception):
                    print(f"Error getting stats for queue {queues[i]}: {summary}")
                    continue
                    
                total_workers += summary.get('total_workers', 0)
                online_workers += summary.get('online_workers', 0)
                offline_workers += summary.get('offline_workers', 0)
                total_success += summary.get('total_success_count', 0)
                total_failed += summary.get('total_failed_count', 0)
                total_tasks += summary.get('total_count', 0)
                total_running += summary.get('total_running_tasks', 0)
                
                # 累加平均处理时间（需要根据任务数加权）
                avg_time = summary.get('avg_processing_time', 0)
                task_count = summary.get('total_count', 0)
                if avg_time > 0 and task_count > 0:
                    total_processing_time += avg_time * task_count
                    total_processing_count += task_count
                
                # 累加平均延迟时间
                avg_latency = summary.get('avg_latency_time', 0)
                if avg_latency > 0 and task_count > 0:
                    total_latency_time += avg_latency * task_count
                    total_latency_count += task_count
            
            # 汇总RabbitMQ风格指标
            for i, stats in enumerate(queue_stats):
                if isinstance(stats, Exception):
                    continue
                    
                total_messages += stats.get('messages', 0)
                total_messages_ready += stats.get('messages_ready', 0)
                total_messages_unacknowledged += stats.get('messages_unacknowledged', 0)
                total_consumers += stats.get('consumers', 0)
                
                message_stats = stats.get('message_stats', {})
                total_publish += message_stats.get('publish', 0)
                total_deliver_get += message_stats.get('deliver_get', 0)
                total_ack += message_stats.get('ack', 0)
            
            # 计算全局平均处理时间
            global_avg_processing_time = 0.0
            if total_processing_count > 0:
                global_avg_processing_time = total_processing_time / total_processing_count
            
            # 计算全局平均延迟时间
            global_avg_latency_time = 0.0
            if total_latency_count > 0:
                global_avg_latency_time = total_latency_time / total_latency_count
            
            return {
                # 原有指标
                'total_queues': len(queues),
                'total_workers': total_workers,
                'online_workers': online_workers,
                'offline_workers': offline_workers,
                'total_success_count': total_success,
                'total_failed_count': total_failed,
                'total_count': total_tasks,
                'total_running_tasks': total_running,
                'avg_processing_time': round(global_avg_processing_time, 3),
                'avg_latency_time': round(global_avg_latency_time, 3),
                'history_included': False,
                # RabbitMQ风格指标
                'messages': total_messages,
                'messages_ready': total_messages_ready,
                'messages_unacknowledged': total_messages_unacknowledged,
                'consumers': total_consumers,
                'message_stats': {
                    'publish': total_publish,
                    'deliver_get': total_deliver_get,
                    'ack': total_ack
                },
                'timestamp': datetime.now(timezone.utc).isoformat()
            }
            
        except Exception as e:
            print(f"Error getting global stats: {e}")
            return {
                'total_queues': 0,
                'total_workers': 0,
                'online_workers': 0,
                'offline_workers': 0,
                'total_success_count': 0,
                'total_failed_count': 0,
                'total_count': 0,
                'total_running_tasks': 0,
                'avg_processing_time': 0.0,
                'avg_latency_time': 0.0,
                'history_included': False,
                'messages': 0,
                'messages_ready': 0,
                'messages_unacknowledged': 0,
                'consumers': 0,
                'message_stats': {
                    'publish': 0,
                    'deliver_get': 0,
                    'ack': 0
                },
                'error': str(e)
            }
        
    async def get_all_queues(self) -> List[str]:
        """获取所有队列名称 - 优先从global:queues集合获取，带缓存"""
        try:
            # 检查缓存是否有效
            current_time = time.time()
            if self._queues_cache is not None and (current_time - self._queues_cache_time) < self._queues_cache_ttl:
                return self._queues_cache
            
            # 优先尝试从全局队列集合获取
            global_queues_key = f'{self.redis_prefix}:global:queues'
            queues = await self.redis.smembers(global_queues_key)
            
            if queues:
                # 如果有全局队列集合，直接使用
                result = sorted(list(queues))
                self._queues_cache = result
                self._queues_cache_time = current_time
                return result
            
            # 如果没有全局队列集合，回退到扫描方式
            queues = set()
            
            # 优化：更精确的扫描模式，只扫描QUEUE:*键
            pattern = f"{self.redis_prefix}:QUEUE:*"
            cursor = 0
            
            # 使用pipeline批量检查类型
            while True:
                cursor, keys = await self.redis.scan(cursor, match=pattern, count=500)  # 增加扫描批次大小
                
                if keys:
                    # 批量检查键类型
                    pipe = self.redis.pipeline()
                    for key in keys:
                        pipe.type(key)
                    
                    types = await pipe.execute()
                    
                    for i, key in enumerate(keys):
                        if types[i] == 'stream':
                            # 提取队列名（去掉前缀）
                            if key.startswith(f"{self.redis_prefix}:QUEUE:"):
                                queue_name = key[len(f"{self.redis_prefix}:QUEUE:"):]
                                queues.add(queue_name)
                
                if cursor == 0:
                    break
            
            # 返回排序后的队列列表并更新缓存
            result = sorted(list(queues))
            self._queues_cache = result
            self._queues_cache_time = current_time
            return result
            
        except Exception as e:
            print(f"Error scanning queues: {e}")
            return []
        
    async def get_queue_stats(self, queue_name: str) -> Dict[str, Any]:
        """获取队列统计信息 (RabbitMQ兼容格式)"""
        prefixed_queue_name = self.get_prefixed_queue_name(queue_name)
        
        try:
            info = await self.redis.xinfo_stream(prefixed_queue_name)
            groups = await self.redis.xinfo_groups(prefixed_queue_name)
        except Exception as e:
            # 如果队列不存在，返回默认值
            return {
                "queue": queue_name,
                "messages": 0,
                "messages_ready": 0, 
                "messages_unacknowledged": 0,
                "consumers": 0,
                "message_stats": {
                    "publish": 0,
                    "deliver_get": 0,
                    "ack": 0
                },
                "consumer_groups": [],
                "error": str(e)
            }
        
        # 计算基础指标
        total_messages = info["length"]
        total_pending = 0
        total_consumers = 0
        total_delivered = 0
        
        consumer_groups_info = []
        
        for group in groups:
            group_pending = group["pending"]
            group_consumers_count = group["consumers"]
            
            total_pending += group_pending
            total_consumers += group_consumers_count
            
            group_info = {
                "name": group["name"],
                "consumers": group_consumers_count,
                "pending": group_pending,
                "last_delivered_id": group["last-delivered-id"]
            }
            
            # 获取消费者详情
            try:
                consumers = await self.redis.xinfo_consumers(prefixed_queue_name, group["name"])
                group_info["consumer_details"] = consumers
                
                # 从消费者统计中计算deliver和ack数量
                for consumer in consumers:
                    total_delivered += consumer.get("pel-count", 0)
                    
            except Exception as e:
                group_info["consumer_details"] = []
                print(f"Error getting consumers for group {group['name']}: {e}")
            
            consumer_groups_info.append(group_info)
        
        # 从Worker统计中获取更精确的消息统计
        worker_summary = await self.get_queue_worker_summary_fast(queue_name)
        publish_count = worker_summary.get('total_count', 0)  # 总处理数作为发布数的近似
        deliver_count = worker_summary.get('total_success_count', 0) + worker_summary.get('total_failed_count', 0)
        ack_count = worker_summary.get('total_success_count', 0)
        
        # 计算就绪消息数 (队列总长度 - 未确认消息数)
        messages_ready = max(0, total_messages - total_pending)
        
        # RabbitMQ风格的统计信息
        stats = {
            "queue": queue_name,
            # RabbitMQ兼容指标
            "messages": total_messages,  # 队列中消息总数
            "messages_ready": messages_ready,  # 就绪状态的消息数
            "messages_unacknowledged": total_pending,  # 未确认的消息数
            "consumers": total_consumers,  # 消费者数量
            "message_stats": {
                "publish": publish_count,  # 发布到队列的消息数量
                "deliver_get": deliver_count,  # 被消费的消息数量
                "ack": ack_count  # 被确认的消息数量
            },
            # 原有详细信息保持兼容性
            "length": info["length"],
            "first_entry": info.get("first-entry"),
            "last_entry": info.get("last-entry"),
            "consumer_groups": consumer_groups_info,
            # 额外的性能指标
            "performance_stats": {
                "avg_processing_time": worker_summary.get('avg_processing_time', 0.0),
                "avg_latency_time": worker_summary.get('avg_latency_time', 0.0),
                "total_running_tasks": worker_summary.get('total_running_tasks', 0)
            }
        }
        
        return stats
    
    async def _heartbeat_scanner(self):
        """心跳扫描器任务，定期检查worker心跳状态"""
        logger = logging.getLogger('webui.heartbeat')
        logger.info("心跳扫描器启动")
        
        while self._scanner_running:
            try:
                # 扫描所有worker
                pattern = f"{self.redis_prefix}:WORKER:*"
                cursor = 0
                worker_keys = []
                
                while True:
                    cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)
                    # 过滤掉HISTORY相关的键
                    for key in keys:
                        if ':HISTORY:' not in key:
                            worker_keys.append(key)
                    if cursor == 0:
                        break
                
                if worker_keys:
                    # 批量获取worker数据
                    pipe = self.redis.pipeline()
                    for key in worker_keys:
                        pipe.hgetall(key)
                    all_workers_data = await pipe.execute()
                    
                    current_time = time.time()
                    
                    # 检查每个worker的心跳
                    for i, worker_data in enumerate(all_workers_data):
                        if not worker_data:
                            continue
                        
                        try:
                            # 获取心跳相关信息
                            last_heartbeat = float(worker_data.get('last_heartbeat', 0))
                            is_alive = worker_data.get('is_alive', 'true').lower() == 'true'
                            heartbeat_timeout = float(worker_data.get('heartbeat_timeout', self.default_heartbeat_timeout))
                            consumer_id = worker_data.get('consumer_id', '')
                            
                            # 检查是否超时
                            if is_alive and (current_time - last_heartbeat) > heartbeat_timeout:
                                logger.info(f"Worker {consumer_id} 心跳超时，标记为离线")
                                
                                # 更新worker状态为离线
                                worker_key = worker_keys[i]
                                await self.redis.hset(worker_key, 'is_alive', 'false')
                                
                        except Exception as e:
                            logger.error(f"检查worker心跳时出错: {e}")
                
                # 等待下一次扫描
                await asyncio.sleep(self.scanner_interval)
                
            except asyncio.CancelledError:
                logger.info("心跳扫描器收到取消信号")
                break
            except Exception as e:
                logger.error(f"心跳扫描器出错: {e}")
                await asyncio.sleep(self.scanner_interval)
        
        logger.info("心跳扫描器已停止")
    
    async def start_heartbeat_scanner(self):
        """启动心跳扫描器"""
        if not self._scanner_running:
            self._scanner_running = True
            self.scanner_task = asyncio.create_task(self._heartbeat_scanner())
            logging.getLogger('webui').info("心跳扫描器任务已创建")
    
    async def stop_heartbeat_scanner(self):
        """停止心跳扫描器"""
        self._scanner_running = False
        if self.scanner_task and not self.scanner_task.done():
            self.scanner_task.cancel()
            try:
                await self.scanner_task
            except asyncio.CancelledError:
                pass

# 创建全局监控器实例
monitor = RedisMonitor()
pg_consumer = None

@asynccontextmanager
async def lifespan(app: FastAPI):
    global pg_consumer
    
    # Startup
    await monitor.connect()
    # 启动心跳扫描器
    await monitor.start_heartbeat_scanner()
    
    # 启动PostgreSQL消费者（如果配置了且显式启用）
    if hasattr(app.state, 'pg_config') and getattr(app.state, 'enable_consumer', False):
        redis_config = RedisConfig.from_env()
        pg_consumer = PostgreSQLConsumer(app.state.pg_config, redis_config)
        await pg_consumer.start()
        logging.info("PostgreSQL consumer started")
    else:
        logging.info("PostgreSQL consumer disabled (use --with-consumer to enable)")
    
    yield
    
    # Shutdown
    await monitor.stop_heartbeat_scanner()
    await monitor.close()
    
    # 停止PostgreSQL消费者
    if pg_consumer:
        await pg_consumer.stop()
    
    # 关闭SQLAlchemy引擎
    global async_engine
    if async_engine:
        await async_engine.dispose()
        async_engine = None
        logger.info("SQLAlchemy async engine closed")

app = FastAPI(title="Jettask Monitor", lifespan=lifespan)

@app.get("/api/tasks")
async def get_tasks():
    """获取所有任务"""
    tasks = await monitor.get_all_tasks()
    return {"tasks": tasks}

@app.get("/api/queue/{queue_name}/tasks")
async def get_queue_tasks(
    queue_name: str, 
    start_time: Optional[str] = None,
    end_time: Optional[str] = None,
    limit: int = 50
):
    """获取指定队列的任务（基于时间范围）"""
    print(f'{queue_name=} {start_time=} {end_time=} {limit=}')
    result = await monitor.get_queue_tasks(queue_name, start_time, end_time, limit)
    return result

@app.get("/api/queue/{queue_name}/timeline/pg")
async def get_queue_timeline_from_pg(
    queue_name: str,
    start_time: Optional[str] = None,
    end_time: Optional[str] = None,
    interval: str = "5m"
):
    """从PostgreSQL获取队列任务的时间分布数据"""
    # 如果没有提供时间范围，默认最近1小时
    if not end_time:
        end_dt = datetime.now(timezone.utc)
    else:
        end_dt = parse_iso_datetime(end_time)
    
    if not start_time:
        start_dt = end_dt - timedelta(hours=1)
    else:
        start_dt = parse_iso_datetime(start_time)
    
    # 解析时间间隔
    interval_minutes = 5  # 默认5分钟
    if interval.endswith('m'):
        interval_minutes = int(interval[:-1])
    elif interval.endswith('h'):
        interval_minutes = int(interval[:-1]) * 60
    
    # 获取数据库引擎
    engine = await get_db_engine()
    if not engine:
        return {
            "timeline": [],
            "interval": interval,
            "start_time": start_dt.isoformat(),
            "end_time": end_dt.isoformat(),
            "error": "PostgreSQL connection not configured"
        }
    
    try:
        async with AsyncSessionLocal() as session:
            # 使用 SQLAlchemy 的原生 SQL 查询（因为复杂的时间分组）
            query = text(f"""
            SELECT 
                DATE_TRUNC('minute', created_at) - 
                INTERVAL '{interval_minutes} minutes' * (EXTRACT(MINUTE FROM created_at)::int % {interval_minutes}) as time_bucket,
                COUNT(*) as count,
                SUM(CASE WHEN status = 'completed' THEN 1 ELSE 0 END) as completed_count,
                SUM(CASE WHEN status = 'failed' THEN 1 ELSE 0 END) as failed_count,
                AVG(CASE WHEN status = 'completed' AND processing_time IS NOT NULL 
                    THEN processing_time ELSE NULL END) as avg_processing_time
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)
            
            result = await session.execute(query, {
                'queue_name': queue_name,
                'start_dt': start_dt,
                'end_dt': end_dt
            })
            rows = result.mappings().all()  # Use mappings() to get dict-like results
            
            # 构建时间轴数据
            timeline = []
            for row in rows:
                timeline.append({
                    "time": row['time_bucket'].isoformat(),
                    "count": row['count'],
                    "completed_count": row['completed_count'],
                    "failed_count": row['failed_count'],
                    "avg_processing_time": float(row['avg_processing_time']) if row['avg_processing_time'] else 0
                })
            
            # 填充缺失的时间点
            filled_timeline = []
            current_time = start_dt
            timeline_dict = {item['time']: item for item in timeline}
            
            while current_time < end_dt:
                time_key = current_time.isoformat()
                if time_key in timeline_dict:
                    filled_timeline.append(timeline_dict[time_key])
                else:
                    filled_timeline.append({
                        "time": time_key,
                        "count": 0,
                        "completed_count": 0,
                        "failed_count": 0,
                        "avg_processing_time": 0
                    })
                current_time += timedelta(minutes=interval_minutes)
            
            return {
                "timeline": filled_timeline,
                "interval": interval,
                "start_time": start_dt.isoformat(),
                "end_time": end_dt.isoformat()
            }
            
    except Exception as e:
        logger.error(f"Error fetching timeline from PostgreSQL: {e}")
        return {
            "timeline": [],
            "interval": interval,
            "start_time": start_dt.isoformat(),
            "end_time": end_dt.isoformat(),
            "error": str(e)
        }

@app.get("/api/queues/timeline/pg")
async def get_queues_timeline_from_pg(
    queues: str = Query(..., description="Comma-separated list of queue names"),
    start_time: Optional[str] = None,
    end_time: Optional[str] = None
):
    """从PostgreSQL获取多个队列的任务时间分布数据"""
    # 解析队列列表
    if not queues or queues.strip() == "":
        # 如果没有提供队列，返回空结果
        # 计算默认时间范围
        end_dt = datetime.now(timezone.utc) if not end_time else parse_iso_datetime(end_time)
        start_dt = (end_dt - timedelta(hours=1)) if not start_time else parse_iso_datetime(start_time)
        
        return {
            "queues": [],
            "start_time": start_dt.isoformat(),
            "end_time": end_dt.isoformat(),
            "interval": interval,
            "message": "No queues selected"
        }
    
    queue_list = [q.strip() for q in queues.split(',') if q.strip()][:10]  # 最多10个队列
    
    # 如果没有提供时间范围，默认最近1小时
    if not end_time:
        end_dt = datetime.now(timezone.utc)
    else:
        end_dt = parse_iso_datetime(end_time)
    
    if not start_time:
        start_dt = end_dt - timedelta(hours=1)
    else:
        start_dt = parse_iso_datetime(start_time)
    
    logger.info(f'{start_dt=} {end_dt=}')
    
    # 根据时间范围自动计算合适的时间间隔
    duration = (end_dt - start_dt).total_seconds()
    
    # 动态计算时间间隔
    if duration <= 300:  # <= 5分钟
        interval_seconds = 0.5  # 500毫秒
        interval_type = 'millisecond'
        interval = '500ms'
    elif duration <= 900:  # <= 15分钟
        interval_seconds = 1  # 1秒
        interval_type = 'second'
        interval = '1s'
    elif duration <= 1800:  # <= 30分钟
        interval_seconds = 2  # 2秒
        interval_type = 'second'
        interval = '2s'
    elif duration <= 3600:  # <= 1小时
        interval_seconds = 30  # 30秒
        interval_type = 'second'
        interval = '30s'
    elif duration <= 10800:  # <= 3小时
        interval_seconds = 300  # 5分钟
        interval_type = 'minute'
        interval = '5m'
    elif duration <= 21600:  # <= 6小时
        interval_seconds = 600  # 10分钟
        interval_type = 'minute'
        interval = '10m'
    elif duration <= 43200:  # <= 12小时
        interval_seconds = 1800  # 30分钟
        interval_type = 'minute'
        interval = '30m'
    elif duration <= 86400:  # <= 24小时
        interval_seconds = 3600  # 1小时
        interval_type = 'hour'
        interval = '1h'
    elif duration <= 172800:  # <= 2天
        interval_seconds = 7200  # 2小时
        interval_type = 'hour'
        interval = '2h'
    elif duration <= 604800:  # <= 7天
        interval_seconds = 21600  # 6小时
        interval_type = 'hour'
        interval = '6h'
    else:  # > 7天
        interval_seconds = 86400  # 1天
        interval_type = 'hour'
        interval = '24h'
    
    # 转换为分钟数（用于兼容旧代码）
    interval_minutes = interval_seconds / 60
    
    logger.info(f"Time range: {duration}s, using interval: {interval} -> {interval_seconds} seconds, type: {interval_type}")
    
    # 获取数据库引擎
    engine = await get_db_engine()
    if not engine:
        return {
            "queues": [],
            "start_time": start_dt.isoformat(),
            "end_time": end_dt.isoformat(),
            "interval": interval,
            "error": "PostgreSQL connection not configured"
        }
    
    result = []
    
    for queue_name in queue_list:
        try:
            async with AsyncSessionLocal() as session:
                # 使用更简单直接的时间分组方法
                if interval_type == 'millisecond':
                    # 对于毫秒级别的间隔
                    query = text(f"""
                    SELECT 
                        DATE_TRUNC('second', created_at) + 
                        INTERVAL '{interval_seconds} seconds' * FLOOR(EXTRACT(MILLISECONDS FROM created_at) / ({interval_seconds} * 1000)) as time_bucket,
                        COUNT(*) as count
                    FROM tasks
                    WHERE queue_name = :queue_name
                        AND created_at >= :start_dt
                        AND created_at < :end_dt
                    GROUP BY time_bucket
                    ORDER BY time_bucket
                    """)
                elif interval_type == 'second':
                    # 对于秒级别的间隔
                    query = text(f"""
                    SELECT 
                        DATE_TRUNC('minute', created_at) + 
                        INTERVAL '{interval_seconds} seconds' * FLOOR(EXTRACT(SECOND FROM created_at) / {interval_seconds}) as time_bucket,
                        COUNT(*) as count
                    FROM tasks
                    WHERE queue_name = :queue_name
                        AND created_at >= :start_dt
                        AND created_at < :end_dt
                    GROUP BY time_bucket
                    ORDER BY time_bucket
                    """)
                elif interval_type == 'minute' and interval_minutes < 60:
                    # 对于分钟级别的间隔（小于1小时）
                    query = text(f"""
                    SELECT 
                        DATE_TRUNC('hour', created_at) + 
                        INTERVAL '{interval_minutes} minutes' * FLOOR(EXTRACT(MINUTE FROM created_at) / {interval_minutes}) as time_bucket,
                        COUNT(*) as count
                    FROM tasks
                    WHERE queue_name = :queue_name
                        AND created_at >= :start_dt
                        AND created_at < :end_dt
                    GROUP BY time_bucket
                    ORDER BY time_bucket
                    """)
                elif interval_minutes == 60:
                    # 对于1小时间隔，直接使用小时截断
                    query = text("""
                    SELECT 
                        DATE_TRUNC('hour', created_at) as time_bucket,
                        COUNT(*) as count
                    FROM tasks
                    WHERE queue_name = :queue_name
                        AND created_at >= :start_dt
                        AND created_at < :end_dt
                    GROUP BY time_bucket
                    ORDER BY time_bucket
                    """)
                else:
                    # 对于大于1小时的间隔，使用小时级别的计算
                    interval_hours = int(interval_minutes // 60)
                    query = text(f"""
                    SELECT 
                        DATE_TRUNC('day', created_at) + 
                        INTERVAL '{interval_hours} hours' * FLOOR(EXTRACT(HOUR FROM created_at) / {interval_hours}) as time_bucket,
                        COUNT(*) as count
                    FROM tasks
                    WHERE queue_name = :queue_name
                        AND created_at >= :start_dt
                        AND created_at < :end_dt
                    GROUP BY time_bucket
                    ORDER BY time_bucket
                    """)
                params = {
                    'queue_name': queue_name,
                    'start_dt': start_dt,
                    'end_dt': end_dt
                }

                # 先绑定参数
                bound_query = query.bindparams(**params)

                # 生成可直接执行的 SQL（带参数值）
                compiled_sql = bound_query.compile(
                    dialect=postgresql.dialect(),
                    compile_kwargs={"literal_binds": True}
                ).string
                compiled_sql = compiled_sql.replace("%%", "%")

                print("可直接复制到 Navicat 执行的 SQL：\n", compiled_sql)

                # 再执行
                result_obj = await session.execute(query, params)
                rows = result_obj.mappings().all()  # Use mappings() to get dict-like results
                logger.info(f'{rows=}')
                # 构建时间轴数据
                timeline = []
                for row in rows:
                    timeline.append({
                        "time": row['time_bucket'].isoformat(),
                        "count": row['count']
                    })
                
                # 填充缺失的时间点
                filled_timeline = []
                
                # 构建一个时间到数据的映射，用于快速查找
                # 由于可能存在时区或微小时间差异，我们需要更灵活的匹配
                timeline_data = []
                for item in timeline:
                    dt = datetime.fromisoformat(item['time'])
                    timeline_data.append((dt, item['count']))
                
                # 对timeline_data按时间排序
                timeline_data.sort(key=lambda x: x[0])
                
                # 生成完整的时间序列
                filled_timeline = []
                
                # 对齐到interval
                def align_to_interval(dt, interval_seconds):
                    """对齐时间到interval_seconds的整数倍"""
                    if interval_seconds >= 3600:  # 大于等于1小时
                        # 按小时对齐
                        dt = dt.replace(minute=0, second=0, microsecond=0)
                        interval_hours = interval_seconds // 3600
                        aligned_hour = (dt.hour // interval_hours) * interval_hours
                        return dt.replace(hour=aligned_hour)
                    elif interval_seconds >= 60:  # 大于等于1分钟
                        # 按分钟对齐
                        dt = dt.replace(second=0, microsecond=0)
                        interval_minutes = interval_seconds // 60
                        total_minutes = dt.hour * 60 + dt.minute
                        aligned_total_minutes = (total_minutes // interval_minutes) * interval_minutes
                        aligned_hour = aligned_total_minutes // 60
                        aligned_minute = aligned_total_minutes % 60
                        return dt.replace(hour=aligned_hour, minute=aligned_minute)
                    elif interval_seconds >= 1:  # 秒级别
                        # 按秒对齐
                        dt = dt.replace(microsecond=0)
                        aligned_second = int(dt.second // interval_seconds) * int(interval_seconds)
                        return dt.replace(second=aligned_second)
                    else:  # 毫秒级别
                        # 按毫秒对齐
                        total_ms = dt.microsecond / 1000  # 转换为毫秒
                        interval_ms = interval_seconds * 1000
                        aligned_ms = int(total_ms // interval_ms) * interval_ms
                        aligned_microsecond = int(aligned_ms * 1000)
                        return dt.replace(microsecond=aligned_microsecond)
                
                current_time = align_to_interval(start_dt, interval_seconds)
                
                # 用于追踪我们在timeline_data中的位置
                timeline_index = 0
                
                while current_time < end_dt:
                    # 查找是否有匹配的数据点
                    # 允许最多interval_seconds/2的误差
                    tolerance = timedelta(seconds=interval_seconds/2)
                    found = False
                    
                    # 从当前位置开始查找
                    while timeline_index < len(timeline_data):
                        data_time, count = timeline_data[timeline_index]
                        
                        # 计算时间差（秒）
                        time_diff = abs((data_time - current_time).total_seconds())
                        
                        if time_diff < interval_seconds / 2:
                            # 找到匹配的数据
                            filled_timeline.append({
                                "time": current_time.isoformat(),
                                "count": count
                            })
                            found = True
                            timeline_index += 1
                            break
                        elif data_time > current_time + tolerance:
                            # 数据时间已经超过当前时间太多，停止查找
                            break
                        else:
                            # 这个数据点太早了，继续查找下一个
                            timeline_index += 1
                    
                    if not found:
                        # 没有找到匹配的数据，填充0
                        filled_timeline.append({
                            "time": current_time.isoformat(),
                            "count": 0
                        })
                    
                    current_time += timedelta(seconds=interval_seconds)
                result.append({
                    "queue": queue_name,
                    "timeline": {
                        "timeline": filled_timeline,
                        "interval": interval
                    }
                })
                
        except Exception as e:
            logger.error(f"Error fetching timeline for queue {queue_name}: {e}")
            result.append({
                "queue": queue_name,
                "timeline": {
                    "timeline": [],
                    "interval": interval,
                    "error": str(e)
                }
            })
    # logger.info(f'{result=}')
    return {
        "queues": result,
        "start_time": start_dt.isoformat(),
        "end_time": end_dt.isoformat(),
        "interval": interval
    }

@app.get("/api/queue/{queue_name}/timeline")
async def get_queue_timeline(
    queue_name: str, 
    interval: str = "1m", 
    duration: str = "1h",
    start_time: Optional[str] = None,
    end_time: Optional[str] = None,
    context: str = "detail"  # 'overview' for homepage, 'detail' for queue detail page
):
    """获取队列任务的时间分布（用于时间轴）"""
    try:
        # 解析时间间隔和持续时间
        interval_seconds = parse_time_duration(interval)
        
        # 根据上下文设置不同的数据限制
        if context == "overview":
            # 首页概览：固定获取最近1小时的所有数据
            duration_seconds = 3600  # 1小时
            now = int(datetime.now(timezone.utc).timestamp() * 1000)
            start = now - duration_seconds * 1000
            min_id = f"{start}-0"
            max_id = "+"
            max_count = 100000  # 首页概览获取所有数据
        else:
            # 队列详情页：根据参数获取，但限制最多10000条
            if start_time and end_time:
                # 使用提供的时间范围
                min_id = start_time
                max_id = end_time if end_time != '+' else '+'
            else:
                # 使用duration参数计算时间范围
                duration_seconds = parse_time_duration(duration)
                now = int(datetime.now(timezone.utc).timestamp() * 1000)
                start = now - duration_seconds * 1000
                min_id = f"{start}-0"
                max_id = "+"
            max_count = 10000  # 详情页限制10000条
        
        # 获取指定时间范围内的消息
        prefixed_queue_name = monitor.get_prefixed_queue_name(queue_name)
        print(f'{prefixed_queue_name=} {min_id=} {max_id=} {max_count=}')
        messages = await monitor.redis.xrange(
            prefixed_queue_name,
            min=min_id,
            max=max_id,
            count=max_count
        )
        
        # 按时间间隔统计任务数量
        buckets = {}
        bucket_size = interval_seconds * 1000  # 转换为毫秒
        
        # 计算实际的时间范围用于生成时间轴
        if start_time and end_time:
            # 从参数中解析时间范围
            if start_time != '-':
                actual_start = int(start_time.split('-')[0])
            else:
                actual_start = int(datetime.now(timezone.utc).timestamp() * 1000) - 86400000  # 默认24小时前
            
            if end_time != '+':
                actual_end = int(end_time.split('-')[0])
            else:
                actual_end = int(datetime.now(timezone.utc).timestamp() * 1000)
        else:
            # 使用duration参数计算的时间范围
            actual_start = start
            actual_end = now
        
        for msg_id, _ in messages:
            # 从消息ID提取时间戳
            timestamp = int(msg_id.split('-')[0])
            bucket_key = (timestamp // bucket_size) * bucket_size
            buckets[bucket_key] = buckets.get(bucket_key, 0) + 1
        
        # 转换为时间序列数据
        timeline_data = []
        current_bucket = (actual_start // bucket_size) * bucket_size
        
        while current_bucket <= actual_end:
            timeline_data.append({
                "timestamp": current_bucket,
                "count": buckets.get(current_bucket, 0)
            })
            current_bucket += bucket_size
        
        # 计算实际任务总数
        total_tasks = len(messages)
        
        # 检查是否达到数据限制
        has_more = False
        if context == "detail" and total_tasks >= max_count:
            has_more = True
        
        return {
            "timeline": timeline_data,
            "interval": interval,
            "duration": duration,
            "start": actual_start,
            "end": actual_end,
            "total_tasks": total_tasks,  # 添加实际任务总数
            "message_count": len(messages),  # 实际获取到的消息数量
            "has_more": has_more,  # 是否还有更多数据
            "limit": max_count if context == "detail" else None  # 数据限制
        }
        
    except Exception as e:
        print(f"Error getting timeline for queue {queue_name}: {e}")
        return {
            "timeline": [],
            "error": str(e)
        }

def parse_time_duration(duration_str: str) -> int:
    """解析时间字符串为秒数 (如 '1h', '10m', '30s')"""
    units = {
        's': 1,
        'm': 60,
        'h': 3600,
        'd': 86400
    }
    
    if duration_str[-1] in units:
        value = int(duration_str[:-1])
        unit = duration_str[-1]
        return value * units[unit]
    
    # 默认为秒
    return int(duration_str)

@app.get("/api/task/{event_id}/result")
async def get_task_result(event_id: str):
    """获取单个任务的结果"""
    result_key = f"{monitor.redis_prefix}:RESULT:{event_id}"
    result = await monitor.redis.get(result_key)
    return {"event_id": event_id, "result": result}

@app.get("/api/queues")
async def get_queues():
    """获取所有队列"""
    queues = await monitor.get_all_queues()
    return {"queues": queues}

@app.get("/api/queue/{queue_name}/stats")
async def get_queue_stats(queue_name: str):
    """获取队列统计信息"""
    try:
        stats = await monitor.get_queue_stats(queue_name)
        return stats
    except Exception as e:
        return {"error": str(e)}

@app.get("/api/queue/{queue_name}/workers")
async def get_queue_workers(queue_name: str):
    """获取队列的Worker信息"""
    workers = await monitor.get_worker_heartbeats(queue_name)
    return {"queue": queue_name, "workers": workers}

@app.get("/api/queue/{queue_name}/worker-summary")
async def get_queue_worker_summary(queue_name: str):
    """获取队列的Worker汇总统计信息"""
    summary = await monitor.get_queue_worker_summary(queue_name)
    return {"queue": queue_name, "summary": summary}

@app.get("/api/workers/offline-history")
async def get_workers_offline_history(
    limit: int = 100,
    start_time: Optional[float] = None,
    end_time: Optional[float] = None
):
    """获取所有worker的下线历史记录"""
    history = await monitor.get_worker_offline_history(limit, start_time, end_time)
    return {"history": history, "total": len(history)}

@app.get("/api/global-stats")
async def get_global_stats():
    """获取全局统计信息（包含历史记录）"""
    stats = await monitor.get_global_stats_with_history()
    return stats

@app.get("/api/global-stats/light")
async def get_global_stats_light():
    """获取轻量级全局统计信息（不包含历史记录）"""
    try:
        # 获取所有队列
        queues = await monitor.get_all_queues()
        
        # 并行获取所有队列的快速汇总和简单统计
        summaries_task = asyncio.gather(
            *[monitor.get_queue_worker_summary_fast(queue) for queue in queues],
            return_exceptions=True
        )
        
        # 获取基础队列信息（不获取完整stats以提高性能）
        queue_lengths_task = asyncio.gather(
            *[monitor.redis.xlen(monitor.get_prefixed_queue_name(queue)) for queue in queues],
            return_exceptions=True
        )
        
        summaries, queue_lengths = await asyncio.gather(
            summaries_task, queue_lengths_task
        )
        
        # 汇总数据
        total_workers = 0
        online_workers = 0
        total_running_tasks = 0
        total_messages = 0
        total_consumers = 0
        
        for summary in summaries:
            if not isinstance(summary, Exception):
                total_workers += summary.get('total_workers', 0)
                online_workers += summary.get('online_workers', 0)
                total_running_tasks += summary.get('total_running_tasks', 0)
                total_consumers += summary.get('total_workers', 0)  # 近似使用worker数作为消费者数
        
        # 汇总消息数
        for length in queue_lengths:
            if not isinstance(length, Exception):
                total_messages += length
        
        return {
            'total_queues': len(queues),
            'total_workers': total_workers,
            'online_workers': online_workers,
            'total_running_tasks': total_running_tasks,
            'messages': total_messages,
            'consumers': total_consumers,
            'timestamp': datetime.now(timezone.utc).isoformat()
        }
    except Exception as e:
        return {
            'error': str(e),
            'total_queues': 0,
            'total_workers': 0,
            'online_workers': 0,
            'total_running_tasks': 0,
            'messages': 0,
            'consumers': 0
        }


@app.get("/api/queue/{queue_name}/workers/offline-history")
async def get_queue_workers_offline_history(
    queue_name: str,
    limit: int = 100,
    start_time: Optional[float] = None,
    end_time: Optional[float] = None
):
    """获取指定队列的worker下线历史记录"""
    # 获取所有历史记录，然后过滤出该队列的
    all_history = await monitor.get_worker_offline_history(limit * 10, start_time, end_time)
    queue_history = [
        record for record in all_history 
        if queue_name in record.get('queues', '').split(',')
    ][:limit]
    return {"queue": queue_name, "history": queue_history, "total": len(queue_history)}

@app.websocket("/ws")
async def websocket_endpoint(websocket: WebSocket):
    """WebSocket端点，用于实时更新（优化版）"""
    await websocket.accept()
    
    try:
        # 标记是否是首次连接
        is_first_load = True
        
        while True:
            try:
                # 检查WebSocket连接状态
                if websocket.client_state != WebSocketState.CONNECTED:
                    break
                    
                # 首次连接时发送基础信息
                if is_first_load:
                    # 只发送队列列表，不发送任务信息
                    data = {
                        "queues": await monitor.get_all_queues(),
                        "timestamp": datetime.now(timezone.utc).isoformat(),
                        "initial_load": True
                    }
                    await websocket.send_json(data)
                    is_first_load = False
                else:
                    # 后续更新：并行获取所有worker信息
                    queues = await monitor.get_all_queues()
                    
                    # 使用 asyncio.gather 并行获取所有队列的worker信息
                    worker_tasks = [monitor.get_worker_heartbeats(queue) for queue in queues]
                    worker_results = await asyncio.gather(*worker_tasks, return_exceptions=True)
                    
                    # 构建队列worker映射
                    queue_workers = {}
                    for i, queue in enumerate(queues):
                        if isinstance(worker_results[i], Exception):
                            print(f"Error getting workers for queue {queue}: {worker_results[i]}")
                            queue_workers[queue] = []
                        else:
                            queue_workers[queue] = worker_results[i]
                    
                    data = {
                        "queues": queues,
                        "workers": queue_workers,
                        "timestamp": datetime.now(timezone.utc).isoformat(),
                        "initial_load": False
                    }
                    
                    # 再次检查连接状态后发送
                    if websocket.client_state == WebSocketState.CONNECTED:
                        await websocket.send_json(data)
                    else:
                        break
                
                await asyncio.sleep(2)  # 每2秒更新一次
                
            except WebSocketDisconnect:
                # WebSocket已断开，退出循环
                break
            except Exception as e:
                # 检查是否是因为连接已关闭导致的错误
                if "close message has been sent" in str(e) or "WebSocket is not connected" in str(e):
                    break
                print(f"Error in websocket loop: {e}")
                # 对于其他错误，等待一段时间后继续
                await asyncio.sleep(5)
            
    except WebSocketDisconnect:
        pass
    except Exception as e:
        print(f"WebSocket error: {e}")
    finally:
        # 确保WebSocket正确关闭
        try:
            await websocket.close()
        except:
            pass

# 挂载静态文件
static_dir = Path(__file__).parent / "static"
static_dir.mkdir(exist_ok=True)

# 挂载静态文件目录
app.mount("/static", StaticFiles(directory=str(static_dir)), name="static")

@app.get("/")
async def read_index():
    """返回主页HTML"""
    html_path = static_dir / "index.html"
    if html_path.exists():
        return HTMLResponse(content=html_path.read_text())
    return HTMLResponse(content="<h1>Jettask Monitor</h1><p>Static files not found</p>")

@app.get("/queue.html")
async def read_queue():
    """返回队列详情页HTML"""
    html_path = static_dir / "queue.html"
    if html_path.exists():
        return HTMLResponse(content=html_path.read_text())
    return HTMLResponse(content="<h1>Queue Details</h1><p>Page not found</p>")

@app.get("/queues.html")
async def read_queues():
    """返回队列列表页HTML"""
    html_path = static_dir / "queues.html"
    if html_path.exists():
        return HTMLResponse(content=html_path.read_text())
    return HTMLResponse(content="<h1>Queues</h1><p>Page not found</p>")

@app.get("/workers.html")
async def read_workers():
    """返回Workers页HTML"""
    html_path = static_dir / "workers.html"
    if html_path.exists():
        return HTMLResponse(content=html_path.read_text())
    return HTMLResponse(content="<h1>Workers</h1><p>Page not found</p>")


# PostgreSQL相关的API端点
@app.get("/api/pg/tasks")
async def get_pg_tasks(
    status: Optional[str] = None,
    queue_name: Optional[str] = None,
    limit: int = 100,
    offset: int = 0
):
    """从PostgreSQL获取任务列表"""
    engine = await get_db_engine()
    if not engine:
        return {"error": "PostgreSQL not configured"}
    
    try:
        async with AsyncSessionLocal() as session:
            # 构建查询
            query = select(Task)
            
            if status:
                query = query.where(Task.status == status)
                
            if queue_name:
                query = query.where(Task.queue_name == queue_name)
                
            query = query.order_by(Task.created_at.desc())
            query = query.limit(limit).offset(offset)
            
            result = await session.execute(query)
            tasks_obj = result.scalars().all()
            
        tasks = []
        for task_obj in tasks_obj:
            task = {
                'id': task_obj.id,
                'queue_name': task_obj.queue_name,
                'task_name': task_obj.task_name,
                'task_data': task_obj.task_data,
                'priority': task_obj.priority,
                'retry_count': task_obj.retry_count,
                'max_retry': task_obj.max_retry,
                'status': task_obj.status,
                'result': task_obj.result,
                'error_message': task_obj.error_message,
                'created_at': task_obj.created_at,
                'started_at': task_obj.started_at,
                'completed_at': task_obj.completed_at,
                'worker_id': task_obj.worker_id,
                'execution_time': task_obj.execution_time,
                'duration': task_obj.duration,
                'metadata': task_obj.task_metadata,
                'next_sync_time': task_obj.next_sync_time,
                'sync_check_count': task_obj.sync_check_count
            }
            # 转换时间戳为ISO格式（确保是 UTC）
            for field in ['created_at', 'started_at', 'completed_at']:
                if task.get(field):
                    # PostgreSQL 的 TIMESTAMP WITH TIME ZONE 会返回 aware datetime
                    if task[field].tzinfo is None:
                        # 如果没有时区信息，假定为 UTC
                        task[field] = task[field].replace(tzinfo=timezone.utc)
                    task[field] = task[field].isoformat()
            # 解析JSON字段
            for field in ['task_data', 'result', 'metadata']:
                if task.get(field) and isinstance(task[field], str):
                    try:
                        task[field] = json.loads(task[field])
                    except:
                        pass
            tasks.append(task)
            
        return {"tasks": tasks, "total": len(tasks)}
        
    except Exception as e:
        logging.error(f"Error fetching tasks from PostgreSQL: {e}")
        return {"error": str(e)}


@app.get("/api/pg/stats")
async def get_pg_stats():
    """获取PostgreSQL中的统计信息"""
    engine = await get_db_engine()
    if not engine:
        return {"error": "PostgreSQL not configured"}
    
    try:
        async with AsyncSessionLocal() as session:
            # 获取任务统计
            task_stats_query = text("""
                SELECT 
                    COUNT(*) as total_tasks,
                    COUNT(CASE WHEN status = 'pending' THEN 1 END) as pending_tasks,
                    COUNT(CASE WHEN status = 'running' THEN 1 END) as running_tasks,
                    COUNT(CASE WHEN status = 'success' THEN 1 END) as completed_tasks,
                    COUNT(CASE WHEN status = 'failed' THEN 1 END) as failed_tasks
                FROM tasks
            """)
            
            task_stats_result = await session.execute(task_stats_query)
            task_stats = task_stats_result.mappings().fetchone()
            
            # 获取队列统计
            queue_stats_query = text("""
                SELECT 
                    queue_name,
                    COUNT(*) as total_tasks,
                    COUNT(CASE WHEN status = 'pending' THEN 1 END) as pending_tasks,
                    COUNT(CASE WHEN status = 'running' THEN 1 END) as running_tasks,
                    COUNT(CASE WHEN status = 'success' THEN 1 END) as completed_tasks,
                    COUNT(CASE WHEN status = 'failed' THEN 1 END) as failed_tasks
                FROM tasks
                GROUP BY queue_name
                ORDER BY total_tasks DESC
            """)
            
            queue_stats_result = await session.execute(queue_stats_query)
            queue_stats = queue_stats_result.mappings().all()
            
        return {
            "task_stats": dict(task_stats) if task_stats else {},
            "queue_stats": [dict(row) for row in queue_stats]
        }
        
    except Exception as e:
        logging.error(f"Error fetching stats from PostgreSQL: {e}")
        return {"error": str(e)}


@app.get("/api/pg/task/{task_id}")
async def get_pg_task(task_id: str):
    """从PostgreSQL获取单个任务的详细信息"""
    engine = await get_db_engine()
    if not engine:
        return {"error": "PostgreSQL not configured"}
    
    try:
        async with AsyncSessionLocal() as session:
            result = await session.execute(select(Task).where(Task.id == task_id))
            task_obj = result.scalar_one_or_none()
            
        if not task_obj:
            return {"error": "Task not found"}
            
        task = {
            'id': task_obj.id,
            'queue_name': task_obj.queue_name,
            'task_name': task_obj.task_name,
            'task_data': task_obj.task_data,
            'priority': task_obj.priority,
            'retry_count': task_obj.retry_count,
            'max_retry': task_obj.max_retry,
            'status': task_obj.status,
            'result': task_obj.result,
            'error_message': task_obj.error_message,
            'created_at': task_obj.created_at,
            'started_at': task_obj.started_at,
            'completed_at': task_obj.completed_at,
            'worker_id': task_obj.worker_id,
            'execution_time': task_obj.execution_time,
            'duration': task_obj.duration,
            'metadata': task_obj.task_metadata,
            'next_sync_time': task_obj.next_sync_time,
            'sync_check_count': task_obj.sync_check_count
        }
        # 转换时间戳为ISO格式（确保是 UTC）
        for field in ['created_at', 'started_at', 'completed_at']:
            if task.get(field):
                # PostgreSQL 的 TIMESTAMP WITH TIME ZONE 会返回 aware datetime
                if task[field].tzinfo is None:
                    # 如果没有时区信息，假定为 UTC
                    task[field] = task[field].replace(tzinfo=timezone.utc)
                task[field] = task[field].isoformat()
        # 解析JSON字段
        for field in ['task_data', 'result', 'metadata']:
            if task.get(field) and isinstance(task[field], str):
                try:
                    task[field] = json.loads(task[field])
                except:
                    pass
                    
        return {"task": task}
        
    except Exception as e:
        logging.error(f"Error fetching task from PostgreSQL: {e}")
        return {"error": str(e)}

if __name__ == "__main__":
    # 配置日志
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    
    uvicorn.run(app, host="0.0.0.0", port=8000)


import os
import sys
import time
from datetime import datetime
from ..utils.serializer import dumps, loads, dumps_str, loads_str
import signal
import socket
import asyncio
import logging
import contextlib
import importlib
import multiprocessing
from typing import List
from collections import defaultdict, deque

import redis
from redis import asyncio as aioredis
from watchdog.observers import Observer

from .task import Task
from .event_pool import EventPool
from ..executors import AsyncioExecutor, MultiAsyncioExecutor
from ..monitoring import FileChangeHandler
from ..utils import gen_task_name
from ..exceptions import TaskTimeoutError, TaskExecutionError, TaskNotFoundError

logger = logging.getLogger('app')

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

# 尝试导入性能优化库
try:
    import uvloop
    UVLOOP_AVAILABLE = True
    # 自动启用uvloop
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    logger.debug("Using uvloop for better performance")
except ImportError:
    UVLOOP_AVAILABLE = False

_on_app_finalizers = set()

# 全局连接池复用
_redis_pools = {}
_async_redis_pools = {}
# 专门用于二进制数据的连接池（用于Stream操作）
_binary_redis_pools = {}
_async_binary_redis_pools = {}

def get_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建Redis连接池"""
    if redis_url not in _redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _redis_pools[redis_url] = redis.ConnectionPool.from_url(
            redis_url, 
            decode_responses=True,
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _redis_pools[redis_url]

def get_async_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建异步Redis连接池"""
    if redis_url not in _async_redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _async_redis_pools[redis_url] = aioredis.ConnectionPool.from_url(
            redis_url,
            decode_responses=True,
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _async_redis_pools[redis_url]

def get_binary_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建用于二进制数据的Redis连接池（不解码响应）"""
    if redis_url not in _binary_redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _binary_redis_pools[redis_url] = redis.ConnectionPool.from_url(
            redis_url, 
            decode_responses=False,  # 不解码，保持二进制
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _binary_redis_pools[redis_url]

def get_async_binary_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建用于二进制数据的异步Redis连接池（不解码响应）"""
    if redis_url not in _async_binary_redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _async_binary_redis_pools[redis_url] = aioredis.ConnectionPool.from_url(
            redis_url,
            decode_responses=False,  # 不解码，保持二进制
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _async_binary_redis_pools[redis_url]


def connect_on_app_finalize(callback):
    """Connect callback to be called when any app is finalized."""
    _on_app_finalizers.add(callback)
    return callback


class Jettask(object):
    # Lua脚本定义为类常量，避免重复定义
    _LUA_SCRIPT_DELAYED_TASKS = """
    local prefix = ARGV[1]
    local current_time = tonumber(ARGV[2])
    local results = {}
    
    -- 从ARGV[3]开始，每5个参数为一组任务信息
    -- [stream_key, stream_data, execute_at, delay_seconds, queue]
    for i = 3, #ARGV, 5 do
        local stream_key = ARGV[i]
        local stream_data = ARGV[i+1]
        local execute_at = tonumber(ARGV[i+2])
        local delay_seconds = tonumber(ARGV[i+3])
        local queue = ARGV[i+4]
        
        -- 1. 添加消息到Stream
        local stream_id = redis.call('XADD', stream_key, '*', 'data', stream_data)
        
        -- 2. 添加到延迟队列ZSET
        local delayed_queue_key = prefix .. ':DELAYED_QUEUE:' .. queue
        redis.call('ZADD', delayed_queue_key, execute_at, stream_id)
        
        -- 3. 设置任务状态Hash（只存储status，其他信息从Stream获取）
        local task_key = prefix .. ':TASK:' .. stream_id
        redis.call('HSET', task_key, 'status', 'delayed')
        
        -- 4. 设置过期时间
        local expire_seconds = math.max(1, math.floor(delay_seconds + 3600))
        redis.call('EXPIRE', task_key, expire_seconds)
        
        -- 保存stream_id到结果
        table.insert(results, stream_id)
    end
    
    return results
    """
    
    _LUA_SCRIPT_NORMAL_TASKS = """
    local prefix = ARGV[1]
    local current_time = ARGV[2]
    local results = {}
    
    -- 从ARGV[3]开始，每2个参数为一组任务信息
    -- [stream_key, stream_data]
    for i = 3, #ARGV, 2 do
        local stream_key = ARGV[i]
        local stream_data = ARGV[i+1]
        
        -- 1. 添加消息到Stream
        local stream_id = redis.call('XADD', stream_key, '*', 'data', stream_data)
        
        -- 2. 设置任务状态Hash（只存储status）
        local task_key = prefix .. ':TASK:' .. stream_id
        redis.call('HSET', task_key, 'status', 'pending')
        
        -- 3. 设置过期时间（1小时）
        redis.call('EXPIRE', task_key, 3600)
        
        -- 保存stream_id到结果
        table.insert(results, stream_id)
    end
    
    return results
    """

    def __init__(self, redis_url: str = None, include: list = None, max_connections: int = 200, 
                 consumer_strategy: str = None, consumer_config: dict = None, tasks=None,
                 redis_prefix: str = None, scheduler_config: dict = None, pg_url: str = None) -> None:
        self._tasks = tasks or {}
        self.asyncio = False
        self.include = include or []
        self.redis_url = redis_url
        self.pg_url = pg_url  # 存储PostgreSQL URL
        self.max_connections = max_connections
        self.consumer_strategy = consumer_strategy
        self.consumer_config = consumer_config or {}
        self.scheduler_config = scheduler_config or {}
        
        # Redis prefix configuration
        self.redis_prefix = redis_prefix or "jettask"
        
        # Update prefixes with the configured prefix using colon namespace
        self.STATUS_PREFIX = f"{self.redis_prefix}:STATUS:"
        self.RESULT_PREFIX = f"{self.redis_prefix}:RESULT:"
        
        # 预编译常用操作，减少运行时开销
        self._loads = loads
        self._dumps = dumps
        
        # 调度器相关
        self.scheduler = None
        self.scheduler_manager = None
        
        self._status_prefix = self.STATUS_PREFIX
        self._result_prefix = self.RESULT_PREFIX
        
        # 初始化清理状态，但不注册处理器
        self._cleanup_done = False
        self._should_exit = False
        self._worker_started = False
        self._handlers_registered = False
    
    def _setup_cleanup_handlers(self):
        """设置清理处理器"""
        # 避免重复注册
        if self._handlers_registered:
            return
        
        self._handlers_registered = True
        
        def signal_cleanup_handler(signum=None, frame=None):
            """信号处理器"""
            if self._cleanup_done:
                return
            # 只有启动过worker才需要打印清理信息
            if self._worker_started:
                logger.info("Received shutdown signal, cleaning up...")
            self.cleanup()
            if signum:
                # 设置标记表示需要退出
                self._should_exit = True
                # 对于多进程环境，不直接操作事件循环
                # 让执行器自己检测退出标志并优雅关闭
        
        def atexit_cleanup_handler():
            """atexit处理器"""
            if self._cleanup_done:
                return
            # atexit时不重复打印日志，静默清理
            self.cleanup()
        
        # 注册信号处理器
        if hasattr(signal, 'SIGTERM'):
            signal.signal(signal.SIGTERM, signal_cleanup_handler)
        if hasattr(signal, 'SIGINT'):
            signal.signal(signal.SIGINT, signal_cleanup_handler)
        
        # 注册atexit处理器
        import atexit
        atexit.register(atexit_cleanup_handler)
    
    def cleanup(self):
        """清理应用资源"""
        if self._cleanup_done:
            return
        
        self._cleanup_done = True
        
        # 只有真正启动过worker才打印日志
        if self._worker_started:
            logger.info("Cleaning up Jettask resources...")
            
            # 清理EventPool
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()
            
            logger.info("Jettask cleanup completed")
        else:
            # 如果只是实例化但没有启动，静默清理
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()
            logger.debug("Jettask instance cleanup (no worker started)")
    
    @property
    def consumer_manager(self):
        """获取消费者管理器"""
        return self.ep.consumer_manager if hasattr(self.ep, 'consumer_manager') else None

    @property
    def async_redis(self):
        """优化：复用连接池"""
        name = "_async_redis"
        if hasattr(self, name):
            return getattr(self, name)
        
        pool = get_async_redis_pool(self.redis_url, self.max_connections)
        async_redis = aioredis.StrictRedis(connection_pool=pool)
        setattr(self, name, async_redis)
        return async_redis

    @property
    def redis(self):
        """优化：复用连接池"""
        name = "_redis"
        if hasattr(self, name):
            return getattr(self, name)
            
        pool = get_redis_pool(self.redis_url, self.max_connections)
        redis_cli = redis.StrictRedis(connection_pool=pool)
        setattr(self, name, redis_cli)
        return redis_cli

    @property
    def ep(self):
        name = "_ep"
        if hasattr(self, name):
            ep = getattr(self, name)
        else:
            # 传递redis_prefix到consumer_config
            consumer_config = self.consumer_config.copy() if self.consumer_config else {}
            consumer_config['redis_prefix'] = self.redis_prefix
            
            ep = EventPool(
                self.redis, 
                self.async_redis, 
                redis_url=self.redis_url,
                consumer_strategy=self.consumer_strategy,
                consumer_config=consumer_config,
                redis_prefix=self.redis_prefix,
                app=self
            )
            setattr(self, name, ep)
        return ep

    def clear(self):
        if hasattr(self, "process"):
            delattr(self, "process")
        if hasattr(self, "_ep"):
            delattr(self, "_ep")

    def get_task_by_name(self, name: str) -> Task:
        # 1. 直接查找完整名称
        task = self._tasks.get(name)
        if task:
            return task
        
        # 2. 如果是简单名称（不含.），尝试匹配所有以该名称结尾的任务
        if '.' not in name:
            for task_key, task_obj in self._tasks.items():
                # 匹配 "module.function_name" 形式，提取函数名部分
                if '.' in task_key:
                    _, func_name = task_key.rsplit('.', 1)
                    if func_name == name:
                        return task_obj
                elif task_key == name:
                    # 完全匹配（可能没有模块前缀）
                    return task_obj
        
        return None

    def include_module(self, modules: list):
        self.include += modules

    def _task_from_fun(
        self, fun, name=None, base=None, queue=None, bind=False, retry_config=None, **options
    ) -> Task:
        name = name or gen_task_name(fun.__name__, fun.__module__)
        base = base or Task
        if name not in self._tasks:
            run = staticmethod(fun)
            task: Task = type(
                fun.__name__,
                (base,),
                dict(
                    {
                        "app": self,
                        "name": name,
                        "run": run,
                        "queue": queue,
                        "retry_config": retry_config,  # 存储重试配置
                        "_decorated": True,
                        "__doc__": fun.__doc__,
                        "__module__": fun.__module__,
                        "__annotations__": fun.__annotations__,
                        "__wrapped__": run,
                    },
                    **options,
                ),
            )()
            task.bind_app(self)
            with contextlib.suppress(AttributeError):
                task.__qualname__ = fun.__qualname__
            self._tasks[task.name] = task
        else:
            task = self._tasks[name]
        return task
    
    def task(
        self,
        name: str = None,
        queue: str = None,
        base: Task = None,
        # 重试相关参数
        max_retries: int = 0,
        retry_backoff: bool = True,  # 是否使用指数退避
        retry_backoff_max: float = 60,  # 最大退避时间（秒）
        retry_on_exceptions: tuple = None,  # 可重试的异常类型
        *args,
        **kwargs,
    ):
        def _create_task_cls(fun):
            # 将重试配置传递给_task_from_fun
            retry_config = None
            if max_retries > 0:
                retry_config = {
                    'max_retries': max_retries,
                    'retry_backoff': retry_backoff,
                    'retry_backoff_max': retry_backoff_max,
                }
                # 将异常类转换为类名字符串，以便序列化
                if retry_on_exceptions:
                    retry_config['retry_on_exceptions'] = [
                        exc if isinstance(exc, str) else exc.__name__ 
                        for exc in retry_on_exceptions
                    ]
            return self._task_from_fun(fun, name, base, queue, retry_config=retry_config, *args, **kwargs)

        return _create_task_cls
    
    def publish_broadcast(
        self,
        queue: str,
        message: dict,
        target_tasks: list = None,
        asyncio_mode: bool = False
    ):
        """
        发布广播消息到队列
        
        Args:
            queue: 目标队列名
            message: 消息内容
            target_tasks: 目标任务列表（None表示所有监听该队列的任务）
            asyncio_mode: 是否使用异步模式
        
        Returns:
            消息ID或协程对象
        
        使用示例：
            # 同步模式
            app.publish_broadcast(
                queue="events",
                message={"type": "customer_registered", "data": {...}}
            )
            
            # 异步模式
            await app.publish_broadcast(
                queue="events",
                message={"type": "order_created", "data": {...}},
                target_tasks=["send_email", "update_inventory"],
                asyncio_mode=True
            )
        """
        # 构建广播消息，直接序列化整个消息
        from ..utils.serializer import dumps_str
        
        # 创建完整的广播消息
        broadcast_message = {
            **message,  # 用户的原始消息
            "_broadcast": True,
            "_target_tasks": target_tasks,
            "_timestamp": time.time(),
            "trigger_time": time.time()
        }
        # 使用EventPool的send_event方法，保持和apply_async一致的格式
        if asyncio_mode:
            return self.ep.send_event(queue, broadcast_message, asyncio=True)
        else:
            return self.ep.send_event(queue, broadcast_message, asyncio=False)
    
    def register_router(self, router, prefix: str = None):
        """
        注册任务路由器
        
        Args:
            router: TaskRouter实例
            prefix: 额外的前缀（可选）
        
        使用示例：
            from jettask import Jettask, TaskRouter
            
            # 创建路由器
            email_router = TaskRouter(prefix="email", queue="emails")
            
            @email_router.task()
            async def send_email(to: str):
                pass
            
            # 注册到主应用
            app = Jettask(redis_url="redis://localhost:6379/0")
            app.register_router(email_router)
        """
        from ..router import TaskRouter
        
        if not isinstance(router, TaskRouter):
            raise TypeError("router must be a TaskRouter instance")
        
        # 注册所有任务
        for task_name, task_config in router.get_tasks().items():
            # 如果指定了额外前缀，添加到任务名
            if prefix:
                if task_config.get('name'):
                    task_config['name'] = f"{prefix}.{task_config['name']}"
                task_name = f"{prefix}.{task_name}"
            
            # 获取任务函数
            func = task_config.pop('func')
            name = task_config.pop('name', task_name)
            queue = task_config.pop('queue', None)
            
            # 注册任务
            task = self._task_from_fun(func, name, None, queue, **task_config)
            logger.info(f"Registered task: {name} (queue: {queue or self.redis_prefix})")
        
        return self

    def _mount_module(self):
        for module in self.include:
            module = importlib.import_module(module)
            for attr_name in dir(module):
                obj = getattr(module, attr_name)
                if hasattr(obj, "app"):
                    self._tasks.update(getattr(obj, "app")._tasks)

    def _validate_tasks_for_executor(self, execute_type: str, queues: List[str]):
        """验证任务类型是否与执行器兼容"""
        if execute_type in ["asyncio", "multi_asyncio"]:
            return  # AsyncIO和MultiAsyncio可以处理异步任务
        
        # 只有Thread执行器不能处理异步任务
        incompatible_tasks = []
        for task_name, task in self._tasks.items():
            # 检查任务是否属于指定队列
            if task.queue not in queues:
                continue
                
            # 检查是否是异步任务
            if asyncio.iscoroutinefunction(task.run):
                incompatible_tasks.append({
                    'name': task_name,
                    'queue': task.queue,
                    'type': 'async'
                })
        
        if incompatible_tasks:
            error_msg = f"\n错误：{execute_type} 执行器不能处理异步任务！\n"
            error_msg += "发现以下异步任务：\n"
            for task in incompatible_tasks:
                error_msg += f"  - {task['name']} (队列: {task['queue']})\n"
            error_msg += f"\n解决方案：\n"
            error_msg += f"1. 使用 asyncio 或 process 执行器\n"
            error_msg += f"2. 或者将这些任务改为同步函数（去掉 async/await）\n"
            error_msg += f"3. 或者将这些任务的队列从监听列表中移除\n"
            raise ValueError(error_msg)
    
    def _start(
        self,
        execute_type: str = "asyncio",
        queues: List[str] = None,
        concurrency: int = 1,
        prefetch_multiplier: int = 1,
        **kwargs
    ):
        # 设置默认队列
        if not queues:
            queues = [self.redis_prefix]
        
        self.ep.queues = queues
        self.ep.init_routing()
        self._mount_module()
        # 验证任务兼容性 
        self._validate_tasks_for_executor(execute_type, queues)
        
        # 收集每个队列上的所有任务（用于广播支持）
        self._tasks_by_queue = {}
        for task_name, task in self._tasks.items():
            task_queue = task.queue or self.redis_prefix
            if task_queue in queues:
                if task_queue not in self._tasks_by_queue:
                    self._tasks_by_queue[task_queue] = []
                self._tasks_by_queue[task_queue].append(task_name)
                logger.debug(f"Task {task_name} listening on queue {task_queue}")
        
        event_queue = deque()
        
        # 创建消费者组
        try:
            self.ep.create_group()
        except Exception as e:
            logger.warning(f"创建消费者组时出错: {e}")
            # 继续执行，listening_event会自动处理
        
        # 根据执行器类型创建对应的执行器
        if execute_type == "asyncio":
            # 对于asyncio执行器，使用asyncio.Queue
            async_event_queue = asyncio.Queue()
            
            async def run_asyncio_executor():
                # 启动异步事件监听
                asyncio.create_task(self.ep.listening_event(async_event_queue, prefetch_multiplier))
                # 创建并运行执行器
                executor = AsyncioExecutor(async_event_queue, self, concurrency)
                await executor.loop()
            
            # try:
            loop = asyncio.get_event_loop()
            # except RuntimeError:
            #     # 如果当前线程没有事件循环，创建一个新的
            #     loop = asyncio.new_event_loop()
            #     asyncio.set_event_loop(loop)
            
            try:
                loop.run_until_complete(run_asyncio_executor())
            except RuntimeError as e:
                if "Event loop stopped" in str(e):
                    logger.info("Event loop stopped, shutting down gracefully")
                else:
                    raise
        elif execute_type == "multi_asyncio":
            # multi_asyncio在每个子进程中会启动自己的监听器
            executor = MultiAsyncioExecutor(event_queue, self, concurrency)
            executor.prefetch_multiplier = prefetch_multiplier
            
            # 设置信号处理器以正确响应Ctrl+C
            def multi_asyncio_signal_handler(signum, _frame):
                logger.info(f"Multi-asyncio mode received signal {signum}")
                executor._main_received_signal = True
                executor.shutdown_event.set()
                # 强制退出主循环
                raise KeyboardInterrupt()
            
            signal.signal(signal.SIGINT, multi_asyncio_signal_handler)
            signal.signal(signal.SIGTERM, multi_asyncio_signal_handler)
            
            try:
                executor.loop()
            except KeyboardInterrupt:
                logger.info("Multi-asyncio mode interrupted")
            finally:
                executor.shutdown()
        else:
            raise ValueError(f"不支持的执行器类型：{execute_type}，仅支持 'asyncio' 和 'multi_asyncio'")

    def _run_subprocess(self, *args, **kwargs):
        # logger.info("Started Worker Process")
        process = multiprocessing.Process(target=self._start, args=args, kwargs=kwargs)
        process.start()
        return process

    def start(
        self,
        execute_type: str = "asyncio",
        queues: List[str] = None,
        concurrency: int = 1,
        prefetch_multiplier: int = 1,
        reload: bool = False,
    ):
        # 标记worker已启动
        self._worker_started = True
        
        # 注册清理处理器（只在启动worker时注册）
        self._setup_cleanup_handlers()
        
        if execute_type == "multi_asyncio" and self.consumer_strategy == "pod":
            raise ValueError("multi_asyncio模式下无法使用pod策略")
        self.process = self._run_subprocess(
            execute_type=execute_type,
            queues=queues,
            concurrency=concurrency,
            prefetch_multiplier=prefetch_multiplier,
        )
        if reload:
            event_handler = FileChangeHandler(
                self,
                execute_type=execute_type,
                queues=queues,
                concurrency=concurrency,
                prefetch_multiplier=prefetch_multiplier,
            )
            observer = Observer()
            observer.schedule(event_handler, ".", recursive=True)
            observer.start()
        # 使用事件来等待，而不是无限循环
        try:
            while not self._should_exit:
                time.sleep(0.1)  # 短暂睡眠，快速响应退出信号
        except KeyboardInterrupt:
            logger.info("Received keyboard interrupt")
            self.cleanup()
            if self.process and self.process.is_alive():
                self.process.terminate()
                self.process.join(timeout=2)
                if self.process.is_alive():
                    logger.warning("Process did not terminate, killing...")
                    self.process.kill()

    def bulk_write(self, tasks: list, asyncio: bool = None):
        """
        统一的批量写入方法，支持同步和异步模式，以及延迟任务
        
        Args:
            tasks: 任务列表
            asyncio: 是否使用异步模式。如果为None，自动检测
        
        Returns:
            同步模式: 返回event_ids列表
            异步模式: 如果在异步环境中调用，返回协程对象
        """
        if not tasks:
            raise ValueError("tasks 参数不能为空！")
        
        # 自动检测异步模式
        if asyncio is None:
            import asyncio as aio
            try:
                loop = aio.get_running_loop()
                asyncio = True
            except RuntimeError:
                asyncio = False
        
        # 如果需要异步执行
        if asyncio:
            return self._bulk_write_async(tasks)
        
        # 同步执行
        import asyncio as aio
        # 在同步模式下，使用 asyncio.run 来运行异步函数
        return aio.run(self._bulk_write_impl(tasks, asyncio_mode=False))
    
    async def _bulk_write_impl(self, tasks: list, asyncio_mode: bool):
        """批量写入的内部实现，支持同步和异步"""
        # 获取对应的Redis客户端
        redis_client = self.get_redis_client(asyncio=asyncio_mode)
        
        # 分离延迟任务和普通任务
        delayed_tasks = []
        normal_tasks = []
        
        for task in tasks:
            if 'delay' in task:
                delayed_tasks.append(task)
            else:
                normal_tasks.append(task)
        
        event_ids = []
        
        # 处理延迟任务 - 使用Lua脚本原子性处理
        if delayed_tasks:
            
            # 准备Lua脚本参数
            current_time = time.time()
            lua_args = [self.redis_prefix, str(current_time)]
            
            for task in delayed_tasks:
                delay_seconds = task.pop('delay')  # 移除delay参数
                queue = task.get("queue")
                
                # 添加执行时间到消息中
                execute_at = current_time + delay_seconds
                task['execute_at'] = execute_at
                task['is_delayed'] = 1  # 使用1代替True
                task['trigger_time'] = current_time  # 添加trigger_time
                
                # 如果queue为None，使用任务名作为队列名
                task_name = task.get("name", "")
                actual_queue = queue or task_name
                task['queue'] = actual_queue
                
                # 准备Stream数据
                prefixed_queue = self.ep.get_prefixed_queue_name(actual_queue)
                from ..utils.serializer import dumps_str
                stream_data = dumps_str(task)
                
                # 添加到Lua脚本参数
                lua_args.extend([
                    prefixed_queue,
                    stream_data,
                    str(execute_at),
                    str(delay_seconds),
                    actual_queue
                ])
            
            # 注册并执行Lua脚本（使用类常量）
            script_attr = '_bulk_delayed_script_async' if asyncio_mode else '_bulk_delayed_script_sync'
            if not hasattr(self, script_attr):
                setattr(self, script_attr, redis_client.register_script(self._LUA_SCRIPT_DELAYED_TASKS))
            
            script = getattr(self, script_attr)
            if asyncio_mode:
                # 异步执行需要await
                stream_ids = await script(keys=[], args=lua_args)
            else:
                stream_ids = script(keys=[], args=lua_args)
            
            event_ids.extend(stream_ids)
        
        # 处理普通任务 - 使用Lua脚本原子性处理
        if normal_tasks:
            
            # 准备Lua脚本参数
            current_time = str(time.time())
            lua_args = [self.redis_prefix, current_time]
            
            # 按队列分组并准备参数
            from ..utils.serializer import dumps_str
            for task in normal_tasks:
                queue = task.get("queue")
                task_name = task.get("name", "")
                actual_queue = queue or task_name
                
                # 准备Stream数据
                prefixed_queue = self.ep.get_prefixed_queue_name(actual_queue)
                stream_data = dumps_str(task)
                
                # 添加到Lua脚本参数
                lua_args.extend([prefixed_queue, stream_data])
            
            # 注册并执行Lua脚本（使用类常量）
            script_attr = '_bulk_normal_script_async' if asyncio_mode else '_bulk_normal_script_sync'
            if not hasattr(self, script_attr):
                setattr(self, script_attr, redis_client.register_script(self._LUA_SCRIPT_NORMAL_TASKS))
            
            script = getattr(self, script_attr)
            if asyncio_mode:
                # 异步执行需要await
                stream_ids = await script(keys=[], args=lua_args)
            else:
                stream_ids = script(keys=[], args=lua_args)
            
            event_ids.extend(stream_ids)
            
        return event_ids

    
    async def _bulk_write_async(self, tasks: list):
        """异步批量写入，直接调用统一的实现"""
        return await self._bulk_write_impl(tasks, asyncio_mode=True)

    def get_task_info(self, event_id: str, asyncio: bool = False):
        """获取任务信息（从TASK:hash）"""
        client = self.get_redis_client(asyncio)
        key = f"{self.redis_prefix}:TASK:{event_id}"
        if asyncio:
            return client.hgetall(key)
        else:
            return client.hgetall(key)
    
    def get_task_status(self, event_id: str, asyncio: bool = False):
        """获取任务状态（从TASK:hash的status字段）"""
        if asyncio:
            return self._get_task_status_async(event_id)
        else:
            client = self.redis
            key = f"{self.redis_prefix}:TASK:{event_id}"
            return client.hget(key, "status")

    async def _get_task_status_async(self, event_id: str):
        """异步获取任务状态"""
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return await self.async_redis.hget(key, "status")

    def set_task_status(self, event_id: str, status: str, asyncio: bool = False):
        """设置任务状态（写入TASK:hash的status字段）"""
        if asyncio:
            return self._set_task_status_async(event_id, status)
        else:
            client = self.redis
            key = f"{self.redis_prefix}:TASK:{event_id}"
            return client.hset(key, "status", status)
    
    async def _set_task_status_async(self, event_id: str, status: str):
        """异步设置任务状态"""
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return await self.async_redis.hset(key, "status", status)

    def set_task_status_by_batch(self, mapping: dict, asyncio: bool = False):
        """批量设置任务状态（写入TASK:hash）"""
        if asyncio:
            return self._set_task_status_by_batch_async(mapping)
        else:
            pipeline = self.redis.pipeline()
            for event_id, status in mapping.items():
                key = f"{self.redis_prefix}:TASK:{event_id}"
                pipeline.hset(key, "status", status)
            return pipeline.execute()
    
    async def _set_task_status_by_batch_async(self, mapping: dict):
        """异步批量设置任务状态"""
        pipeline = self.async_redis.pipeline()
        for event_id, status in mapping.items():
            key = f"{self.redis_prefix}:TASK:{event_id}"
            pipeline.hset(key, "status", status)
        return await pipeline.execute()

    def del_task_status(self, event_id: str, asyncio: bool = False):
        """删除任务状态（删除整个TASK:hash）"""
        client = self.get_redis_client(asyncio)
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return client.delete(key)

    def get_redis_client(self, asyncio: bool = False):
        return self.async_redis if asyncio else self.redis

    def set_data(
        self, event_id: str, result: str, ex: int = 3600, asyncio: bool = False
    ):
        """设置任务结果（写入TASK:hash的result字段）"""
        client = self.get_redis_client(asyncio)
        key = f"{self.redis_prefix}:TASK:{event_id}"
        if asyncio:
            return self._set_data_async(key, result, ex)
        else:
            client.hset(key, "result", result)
            return client.expire(key, ex)
    
    async def _set_data_async(self, key: str, result: str, ex: int):
        """异步设置任务结果"""
        await self.async_redis.hset(key, "result", result)
        return await self.async_redis.expire(key, ex)
    
    async def get_and_delayed_deletion(self, key: str, ex: int):
        """获取结果并延迟删除（从hash中）"""
        result = await self.async_redis.hget(key, "result")
        await self.async_redis.expire(key, ex)
        return result
    
    async def _get_result_async(self, key: str, delete: bool, delayed_deletion_ex: int):
        """异步获取任务结果"""
        client = self.async_redis
        if delayed_deletion_ex is not None:
            result = await client.hget(key, "result")
            await client.expire(key, delayed_deletion_ex)
            return result
        elif delete:
            # 获取结果并删除整个hash
            result = await client.hget(key, "result")
            await client.delete(key)
            return result
        else:
            return await client.hget(key, "result")
    
    def get_result(self, event_id: str, delete: bool = False, asyncio: bool = False, 
                   delayed_deletion_ex: int = None, wait: bool = False, timeout: int = 300,
                   poll_interval: float = 0.5, suppress_traceback: bool = False):
        """获取任务结果（从TASK:hash的result字段）
        
        Args:
            event_id: 任务ID
            delete: 是否删除结果
            asyncio: 是否使用异步模式
            delayed_deletion_ex: 延迟删除时间（秒）
            wait: 是否阻塞等待直到任务完成
            timeout: 等待超时时间（秒），默认300秒
            poll_interval: 轮询间隔（秒），默认0.5秒
            suppress_traceback: 是否抑制框架层堆栈（直接打印错误并退出）
        
        Returns:
            任务结果字符串
            
        Raises:
            TaskTimeoutError: 等待超时
            TaskExecutionError: 任务执行失败
            TaskNotFoundError: 任务不存在
        """
        if asyncio:
            key = f"{self.redis_prefix}:TASK:{event_id}"
            if wait:
                return self._get_result_async_wait(event_id, key, delete, delayed_deletion_ex, timeout, poll_interval)
            else:
                return self._get_result_async(key, delete, delayed_deletion_ex)
        else:
            # 同步模式
            if wait:
                return self._get_result_sync_wait(event_id, delete, delayed_deletion_ex, timeout, poll_interval)
            else:
                client = self.redis
                key = f"{self.redis_prefix}:TASK:{event_id}"
                if delayed_deletion_ex is not None:
                    result = client.hget(key, "result")
                    client.expire(key, delayed_deletion_ex)
                    return result
                elif delete:
                    # 获取结果并删除整个hash
                    result = client.hget(key, "result")
                    client.delete(key)
                    return result
                else:
                    return client.hget(key, "result")
    
    def _get_result_sync_wait(self, event_id: str, delete: bool, delayed_deletion_ex: int, 
                              timeout: int, poll_interval: float):
        """同步模式下阻塞等待任务结果"""
        start_time = time.time()
        
        while True:
            # 检查超时
            if time.time() - start_time > timeout:
                raise TaskTimeoutError(f"Task {event_id} timed out after {timeout} seconds")
            
            # 获取任务状态
            status = self.get_task_status(event_id)
            
            if status is None:
                raise TaskNotFoundError(f"Task {event_id} not found")
            
            if status == 'success':
                # 任务成功，获取结果
                key = f"{self.redis_prefix}:TASK:{event_id}"
                if delayed_deletion_ex is not None:
                    result = self.redis.hget(key, "result")
                    self.redis.expire(key, delayed_deletion_ex)
                    return result
                elif delete:
                    result = self.redis.hget(key, "result")
                    self.redis.delete(key)
                    return result
                else:
                    return self.redis.hget(key, "result")
            
            elif status == 'error':
                # 任务失败，获取错误信息并抛出异常
                key = f"{self.redis_prefix}:TASK:{event_id}"
                # 从 exception 字段获取错误信息
                error_msg = self.redis.hget(key, "exception") or "Task execution failed"
                # 抛出自定义异常
                raise TaskExecutionError(event_id, error_msg)
            
            # 任务还在执行中，继续等待
            time.sleep(poll_interval)
    
    async def _get_result_async_wait(self, event_id: str, key: str, delete: bool, 
                                     delayed_deletion_ex: int, timeout: int, poll_interval: float):
        """异步模式下等待任务结果"""
        start_time = time.time()
        
        while True:
            # 检查超时
            if time.time() - start_time > timeout:
                raise TaskTimeoutError(f"Task {event_id} timed out after {timeout} seconds")
            
            # 获取任务状态
            status = await self._get_task_status_async(event_id)
            
            if status is None:
                raise TaskNotFoundError(f"Task {event_id} not found")
            
            if status == 'success':
                # 任务成功，获取结果
                if delayed_deletion_ex is not None:
                    result = await self.async_redis.hget(key, "result")
                    await self.async_redis.expire(key, delayed_deletion_ex)
                    return result
                elif delete:
                    result = await self.async_redis.hget(key, "result")
                    await self.async_redis.delete(key)
                    return result
                else:
                    return await self.async_redis.hget(key, "result")
            
            elif status == 'error':
                # 任务失败，获取错误信息并抛出异常
                # 从 exception 字段获取错误信息
                error_msg = await self.async_redis.hget(key, "exception") or "Task execution failed"
                # 抛出自定义异常
                raise TaskExecutionError(event_id, error_msg)
            
            # 任务还在执行中，继续等待
            await asyncio.sleep(poll_interval)
    
    # ==================== 定时任务调度相关 ====================
    
    async def _ensure_scheduler_initialized(self, db_url: str = None):
        """确保调度器已初始化（内部方法）"""
        if not self.scheduler_manager:
            logger.info("Auto-initializing scheduler...")
            # 优先使用传入的db_url，然后是实例化时的pg_url，最后是环境变量
            if not db_url:
                db_url = self.pg_url or os.environ.get('JETTASK_PG_URL')
            if not db_url:
                raise ValueError(
                    "Database URL not provided. Please provide pg_url when initializing Jettask, "
                    "or set JETTASK_PG_URL environment variable\n"
                    "Example: app = Jettask(redis_url='...', pg_url='postgresql://...')\n"
                    "Or: export JETTASK_PG_URL='postgresql://user:password@localhost:5432/jettask'"
                )
            
            from ..scheduler import TaskScheduler, ScheduledTaskManager
            
            # 创建数据库管理器
            self.scheduler_manager = ScheduledTaskManager(db_url)
            await self.scheduler_manager.connect()
            await self.scheduler_manager.init_schema()
            
            # 创建调度器
            scheduler_config = self.scheduler_config.copy()
            scheduler_config.setdefault('redis_prefix', f"{self.redis_prefix}:SCHEDULER")
            scheduler_config.setdefault('scan_interval', 0.1)
            scheduler_config.setdefault('batch_size', 100)
            scheduler_config.setdefault('leader_ttl', 10)
            
            self.scheduler = TaskScheduler(
                app=self,
                redis_url=self.redis_url,
                db_manager=self.scheduler_manager,
                **scheduler_config
            )
            
            await self.scheduler.connect()
            logger.info("Scheduler initialized")
    
    async def start_scheduler(self):
        """启动定时任务调度器（自动初始化）"""
        # 自动初始化调度器
        await self._ensure_scheduler_initialized()
        
        try:
            await self.scheduler.run()
        except Exception as e:
            logger.error(f"Scheduler error: {e}", exc_info=True)
            raise
    
    def stop_scheduler(self):
        """停止定时任务调度器"""
        if self.scheduler:
            self.scheduler.stop()
            logger.info("Scheduler stopped")
    
    async def add_scheduled_task(
        self,
        task_name: str,
        scheduler_id: str,  # 必填参数
        queue_name: str = None,
        task_type: str = "interval",
        interval_seconds: float = None,
        cron_expression: str = None,
        task_args: list = None,
        task_kwargs: dict = None,
        next_run_time: datetime = None,
        skip_if_exists: bool = True,
        at_once: bool = True,  # 是否立即保存到数据库
        **extra_params
    ):
        """
        添加定时任务
        
        Args:
            task_name: 要执行的函数名（必须对应已经通过@app.task注册的任务）
            scheduler_id: 任务的唯一标识符（必填，用于去重）
            queue_name: 目标队列名（可选，从task_name对应的任务自动获取）
            task_type: 任务类型 ('once', 'interval', 'cron')
            interval_seconds: 间隔秒数 (task_type='interval'时使用)
            cron_expression: Cron表达式 (task_type='cron'时使用)
            task_args: 任务参数列表
            task_kwargs: 任务关键字参数
            next_run_time: 首次执行时间 (task_type='once'时使用)
            skip_if_exists: 如果任务已存在是否跳过（默认True）
            at_once: 是否立即保存到数据库（默认True），如果False则返回任务对象用于批量写入
            **extra_params: 其他参数 (如 max_retries, timeout, description 等)
        """
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        from ..scheduler.models import ScheduledTask, TaskType
        
        # 尝试从已注册的任务中获取信息
        registered_task = None
        
        # 1. 直接匹配
        registered_task = self._tasks.get(task_name)
        
        # 2. 如果没找到，尝试查找以task_name结尾的任务（如 "module.task_name"）
        if not registered_task:
            for task_key, task_obj in self._tasks.items():
                if task_key.endswith(f".{task_name}") or task_key == task_name:
                    registered_task = task_obj
                    break
        
        if not registered_task:
            # 任务必须已注册
            available_tasks = list(self._tasks.keys())
            error_msg = f"Task '{task_name}' not found in registered tasks.\n"
            error_msg += "All scheduled tasks must be registered with @app.task decorator.\n"
            if available_tasks:
                error_msg += f"Available tasks: {', '.join(available_tasks)}"
            raise ValueError(error_msg)
        
        # 自动填充信息
        if not queue_name:
            queue_name = registered_task.queue or self.redis_prefix
        
        # 使用注册时的完整任务名称（包含模块前缀）
        # 查找任务的完整注册名称
        full_task_name = None
        for task_key, task_obj in self._tasks.items():
            if task_obj == registered_task:
                full_task_name = task_key
                break
        
        if not full_task_name:
            # 如果没找到，使用用户提供的名称
            full_task_name = task_name
        
        # 处理 next_run_time（主要用于 once 类型）
        if task_type == "once" and extra_params.get("next_run_time"):
            # 如果在 extra_params 中，移到正确位置
            next_run_time = extra_params.pop("next_run_time", None)
        
        # 移除不属于ScheduledTask的参数
        extra_params.pop("skip_if_exists", None)
        
        # scheduler_id是必填参数，必须由用户提供
        if not scheduler_id:
            raise ValueError("scheduler_id is required and must be provided")
        
        # 创建任务对象
        task = ScheduledTask(
            scheduler_id=scheduler_id,
            task_name=full_task_name,  # 使用完整的任务名称（包含模块前缀）
            task_type=TaskType(task_type),
            queue_name=queue_name,
            task_args=task_args or [],
            task_kwargs=task_kwargs or {},
            interval_seconds=interval_seconds,
            cron_expression=cron_expression,
            next_run_time=next_run_time,
            **extra_params
        )
        
        # 如果不立即保存，返回任务对象供批量写入
        if not at_once:
            # 设置skip_if_exists标记供批量写入时使用
            task._skip_if_exists = skip_if_exists
            return task
        
        # 保存到数据库（支持去重）
        task, created = await self.scheduler_manager.create_or_get_task(task, skip_if_exists=skip_if_exists)
        
        if created:
            logger.info(f"Scheduled task {task.id} created for function {task_name}")
        else:
            logger.info(f"Scheduled task {task.id} already exists for function {task_name}")
        
        return task
    
    async def remove_scheduled_task(self, scheduler_id: str):
        """移除定时任务"""
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        # 先获取任务
        task = await self.scheduler_manager.get_task_by_scheduler_id(scheduler_id)
        if not task:
            return False
        
        success = await self.scheduler_manager.delete_task(task.id)
        
        if success and self.scheduler:
            # 从Redis中也移除
            await self.scheduler.loader.remove_task(task.id)
        
        return success
    
    async def batch_add_scheduled_tasks(
        self,
        tasks: list,
        skip_existing: bool = True
    ):
        """
        批量添加定时任务
        
        Args:
            tasks: 任务配置列表，每个元素是一个字典，包含add_scheduled_task的参数
            skip_existing: 是否跳过已存在的任务
            
        Returns:
            成功创建的任务列表
        """
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        from ..scheduler.models import ScheduledTask, TaskType
        
        task_objects = []
        for task_config in tasks:
            # 获取任务名称
            task_name = task_config.get('task_name')
            if not task_name:
                logger.warning("Task config missing task_name, skipping")
                continue
            
            # 查找注册的任务
            registered_task = self._tasks.get(task_name)
            if not registered_task:
                for task_key, task_obj in self._tasks.items():
                    if task_key.endswith(f".{task_name}") or task_key == task_name:
                        registered_task = task_obj
                        task_name = task_key  # 使用完整名称
                        break
            
            if not registered_task:
                logger.warning(f"Task '{task_name}' not found in registered tasks, skipping")
                continue
            
            # 准备任务参数
            queue_name = task_config.get('queue_name') or registered_task.queue or self.redis_prefix
            task_type = task_config.get('task_type', 'interval')
            
            # 处理next_run_time
            next_run_time = task_config.get('next_run_time')
            if task_type == 'once' and not next_run_time:
                next_run_time = datetime.now()
            
            # scheduler_id是必填参数
            scheduler_id = task_config.get('scheduler_id')
            if not scheduler_id:
                raise ValueError(f"Task config for '{task_name}' missing required scheduler_id")
            
            # 创建任务对象
            task_obj = ScheduledTask(
                scheduler_id=scheduler_id,
                task_name=task_name,
                task_type=TaskType(task_type),
                queue_name=queue_name,
                task_args=task_config.get('task_args', []),
                task_kwargs=task_config.get('task_kwargs', {}),
                interval_seconds=task_config.get('interval_seconds'),
                cron_expression=task_config.get('cron_expression'),
                next_run_time=next_run_time,
                enabled=task_config.get('enabled', True),
                max_retries=task_config.get('max_retries', 3),
                retry_delay=task_config.get('retry_delay', 60),
                timeout=task_config.get('timeout', 300),
                description=task_config.get('description'),
                tags=task_config.get('tags', []),
                metadata=task_config.get('metadata', {})
            )
            task_objects.append(task_obj)
        
        # 批量创建
        created_tasks = await self.scheduler_manager.batch_create_tasks(task_objects, skip_existing)
        
        logger.info(f"Batch created {len(created_tasks)} scheduled tasks")
        return created_tasks
    
    async def bulk_write_scheduled_tasks(self, tasks: list):
        """
        批量写入定时任务（配合at_once=False使用）
        
        使用示例：
            # 收集任务对象
            tasks = []
            for i in range(100):
                task = await app.add_scheduled_task(
                    task_name="my_task",
                    scheduler_id=f"task_{i}",
                    task_type="interval",
                    interval_seconds=30,
                    at_once=False  # 不立即保存
                )
                tasks.append(task)
            
            # 批量写入
            created_tasks = await app.bulk_write_scheduled_tasks(tasks)
        
        Args:
            tasks: 通过add_scheduled_task(at_once=False)创建的任务对象列表
        
        Returns:
            成功创建的任务列表
        """
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        if not tasks:
            return []
        
        # 准备批量创建的任务列表
        task_objects = []
        for task in tasks:
            if not hasattr(task, 'scheduler_id'):
                logger.warning("Invalid task object, skipping")
                continue
            
            
            task_objects.append(task)
        
        # 批量创建（使用第一个任务的skip_if_exists设置）
        skip_existing = getattr(tasks[0], '_skip_if_exists', True) if tasks else True
        created_tasks = await self.scheduler_manager.batch_create_tasks(task_objects, skip_existing)
        
        logger.info(f"Bulk wrote {len(created_tasks)} scheduled tasks")
        return created_tasks
    
    async def list_scheduled_tasks(self, **filters):
        """列出定时任务"""
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        return await self.scheduler_manager.list_tasks(**filters)
    
    async def get_scheduled_task(self, scheduler_id: str):
        """获取定时任务详情"""
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        return await self.scheduler_manager.get_task_by_scheduler_id(scheduler_id)
    
    async def pause_scheduled_task(self, scheduler_id: str):
        """暂停/禁用定时任务"""
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        # 通过scheduler_id获取任务
        task = await self.scheduler_manager.get_task_by_scheduler_id(scheduler_id)
            
        if task:
            task.enabled = False
            await self.scheduler_manager.update_task(task)
            
            # 从Redis中移除
            if self.scheduler:
                await self.scheduler.loader.remove_task(task.id)
            
            logger.info(f"Task {task.id} (scheduler_id: {task.scheduler_id}) disabled")
            return True
        return False
    
    async def resume_scheduled_task(self, scheduler_id: str):
        """恢复/启用定时任务"""
        # 自动初始化
        await self._ensure_scheduler_initialized()
        
        # 通过scheduler_id获取任务
        task = await self.scheduler_manager.get_task_by_scheduler_id(scheduler_id)
            
        if task:
            task.enabled = True
            task.next_run_time = task.calculate_next_run_time()
            await self.scheduler_manager.update_task(task)
            
            # 触发重新加载
            if self.scheduler:
                await self.scheduler.loader.load_tasks()
            
            logger.info(f"Task {task.id} (scheduler_id: {task.scheduler_id}) enabled")
            return True
        return False
    
    # 别名，更直观
    async def disable_scheduled_task(self, scheduler_id: str):
        """禁用定时任务"""
        return await self.pause_scheduled_task(scheduler_id=scheduler_id)
    
    async def enable_scheduled_task(self, scheduler_id: str):
        """启用定时任务"""
        return await self.resume_scheduled_task(scheduler_id=scheduler_id)
    



"""
Task Context - FastAPI风格的依赖注入
"""

from typing import TYPE_CHECKING, Optional, Any
from dataclasses import dataclass

if TYPE_CHECKING:
    from .app import Jettask


@dataclass
class TaskContext:
    """
    任务上下文信息
    
    通过类型注解自动注入到任务函数中：
    
    @app.task
    async def my_task(ctx: TaskContext, data: dict):
        print(f"Task ID: {ctx.event_id}")
        print(f"Task Name: {ctx.name}")
        return data
    """
    event_id: str
    name: str
    trigger_time: float
    app: "Jettask"
    queue: Optional[str] = None
    worker_id: Optional[str] = None
    retry_count: int = 0
    
    def __repr__(self) -> str:
        return f"TaskContext(event_id={self.event_id}, name={self.name}, queue={self.queue})"


"""数据库初始化工具"""

import asyncio
import logging
import sys
from pathlib import Path

import asyncpg
from asyncpg.exceptions import PostgresError

from jettask.webui.config import PostgreSQLConfig

logger = logging.getLogger(__name__)


class DatabaseInitializer:
    """数据库初始化器"""
    
    def __init__(self, pg_config: PostgreSQLConfig):
        self.pg_config = pg_config
        self.schema_path = Path(__file__).parent / "schema.sql"
        
    async def test_connection(self) -> bool:
        """测试数据库连接"""
        try:
            logger.info(f"正在测试数据库连接: {self.pg_config.host}:{self.pg_config.port}/{self.pg_config.database}")
            
            conn = await asyncpg.connect(self.pg_config.dsn)
            await conn.close()
            
            logger.info("✓ 数据库连接成功")
            return True
            
        except Exception as e:
            logger.error(f"✗ 数据库连接失败: {e}")
            return False
            
    async def create_database(self) -> bool:
        """创建数据库（如果不存在）"""
        try:
            # 连接到默认的postgres数据库
            admin_dsn = f"postgresql://{self.pg_config.user}:{self.pg_config.password}@{self.pg_config.host}:{self.pg_config.port}/postgres"
            conn = await asyncpg.connect(admin_dsn)
            
            # 检查数据库是否存在
            exists = await conn.fetchval(
                "SELECT EXISTS(SELECT 1 FROM pg_database WHERE datname = $1)",
                self.pg_config.database
            )
            
            if not exists:
                logger.info(f"正在创建数据库: {self.pg_config.database}")
                await conn.execute(f'CREATE DATABASE "{self.pg_config.database}"')
                logger.info("✓ 数据库创建成功")
            else:
                logger.info(f"✓ 数据库已存在: {self.pg_config.database}")
                
            await conn.close()
            return True
            
        except Exception as e:
            logger.error(f"✗ 创建数据库失败: {e}")
            logger.info("请确保您有创建数据库的权限，或手动创建数据库")
            return False
            
    async def init_schema(self) -> bool:
        """初始化数据库架构"""
        try:
            if not self.schema_path.exists():
                logger.error(f"✗ Schema文件不存在: {self.schema_path}")
                return False
                
            logger.info("正在读取schema文件...")
            schema_sql = self.schema_path.read_text()
            
            logger.info("正在初始化数据库表结构...")
            conn = await asyncpg.connect(self.pg_config.dsn)
            
            # 执行schema
            await conn.execute(schema_sql)
            
            # 验证表是否创建成功
            tables = await conn.fetch("""
                SELECT tablename FROM pg_tables 
                WHERE schemaname = 'public' 
                AND tablename IN ('tasks', 'queue_stats', 'workers')
                ORDER BY tablename
            """)
            
            created_tables = [row['tablename'] for row in tables]
            logger.info(f"✓ 成功创建表: {', '.join(created_tables)}")
            
            # 显示表结构信息
            for table in created_tables:
                count = await conn.fetchval(f"SELECT COUNT(*) FROM {table}")
                logger.info(f"  - {table}: {count} 条记录")
                
            await conn.close()
            return True
            
        except Exception as e:
            logger.error(f"✗ 初始化schema失败: {e}")
            return False
            
    async def check_permissions(self) -> bool:
        """检查用户权限"""
        try:
            conn = await asyncpg.connect(self.pg_config.dsn)
            
            # 检查基本权限
            permissions = await conn.fetch("""
                SELECT has_table_privilege($1, 'tasks', 'SELECT') as can_select,
                       has_table_privilege($1, 'tasks', 'INSERT') as can_insert,
                       has_table_privilege($1, 'tasks', 'UPDATE') as can_update,
                       has_table_privilege($1, 'tasks', 'DELETE') as can_delete
            """, self.pg_config.user)
            
            if permissions:
                perm = permissions[0]
                logger.info(f"✓ 用户权限检查:")
                logger.info(f"  - SELECT: {'✓' if perm['can_select'] else '✗'}")
                logger.info(f"  - INSERT: {'✓' if perm['can_insert'] else '✗'}")
                logger.info(f"  - UPDATE: {'✓' if perm['can_update'] else '✗'}")
                logger.info(f"  - DELETE: {'✓' if perm['can_delete'] else '✗'}")
                
            await conn.close()
            return True
            
        except Exception as e:
            logger.warning(f"权限检查失败: {e}")
            return True  # 不阻止继续
            
    async def run(self) -> bool:
        """运行完整的初始化流程"""
        logger.info("=" * 50)
        logger.info("开始初始化PostgreSQL数据库")
        logger.info("=" * 50)
        
        # 1. 创建数据库（如果需要）
        if not await self.create_database():
            return False
            
        # 2. 测试连接
        if not await self.test_connection():
            return False
            
        # 3. 初始化schema
        if not await self.init_schema():
            return False
            
        # 4. 检查权限
        await self.check_permissions()
        
        logger.info("=" * 50)
        logger.info("✓ 数据库初始化完成！")
        logger.info("=" * 50)
        logger.info("")
        logger.info("您现在可以启动WebUI:")
        logger.info(f"  python -m jettask.webui --pg-url {self.pg_config.dsn}")
        logger.info("")
        
        return True


async def init_database_async(pg_config: PostgreSQLConfig):
    """初始化数据库的异步入口函数"""
    # 配置日志
    logging.basicConfig(
        level=logging.INFO,
        format='%(message)s',
        handlers=[logging.StreamHandler(sys.stdout)]
    )
    
    initializer = DatabaseInitializer(pg_config)
    success = await initializer.run()
    
    if not success:
        sys.exit(1)

def init_database():
    """初始化数据库的同步入口函数（供 CLI 使用）"""
    import os
    from dotenv import load_dotenv
    
    load_dotenv()
    
    # 从环境变量读取配置
    pg_config = PostgreSQLConfig(
        host=os.getenv('JETTASK_PG_HOST', 'localhost'),
        port=int(os.getenv('JETTASK_PG_PORT', '5432')),
        database=os.getenv('JETTASK_PG_DB', 'jettask'),
        user=os.getenv('JETTASK_PG_USER', 'jettask'),
        password=os.getenv('JETTASK_PG_PASSWORD', '123456'),
    )
    
    asyncio.run(init_database_async(pg_config))